#ifndef __FLASH_H__
#define __FLASH_H__
#include <stdint.h>
#include "common.h"
#define SECTOR_SIZE 512
#define WORD_SIZE 4

typedef enum
{
	FS_OK    =  0,
	FS_ERROR = 1,
}FLASH_STATUS;

#define FMC_FLNVSCNT_chipset_Pos                 (16UL)                     /*!< EVF (Bit 2)                                           */
int EraseChip (uint32_t adr);
int EraseSector (uint32_t adr);
int ProgramPage (uint32_t adr, uint32_t sz, uint8_t *buf);
FLASH_STATUS flash_write(uint32_t adr, uint32_t sz, uint8_t *buf);
void flashRead(uint8_t *read_address, uint16_t size, uint8_t *readbuf);
/***********************************************************************************************************************
* Function Name: flashSoftSetupTime_Set
* @brief  control the program, sector erase and chip erase set up time soft control or hardware control
* @param  psectstat - program and sector erase set up time soft control
*											 ENABLE: enable soft set setup time
*											 DISABLE: enable hardware default setup time
* @param  ChipStTime - chip erase set up time soft control
*											 ENABLE: enable soft set setup time
*											 DISABLE: enable hardware default setup time
* @return none
* @note   
***********************************************************************************************************************/
void flashSoftSetupTime_Set(FunctionalState psectstat,FunctionalState chiperasestat);

/***********************************************************************************************************************
* Function Name: flashSetupTime
* @brief  set the program, sector erase and chip erase set up time
* @param  prgSectErsStTime - program and sector erase set up time(0~4095)
* @param  ChipStTime - chip erase set up time(0~16383)
* @return none
* @note   
***********************************************************************************************************************/
void flashSetupTime(uint16_t prgSectErsStTime,uint16_t ChipStTime);

#endif
