#ifndef __GPIO_H__
#define __GPIO_H__

#include "common.h"
#include <assert.h>

#define GPIO_SUCC  0
#define	GPIO_ERR   1

typedef enum
{ 
	GPIO_Control_DIG   = 0, /*!< GPIO Control Digital */
	GPIO_Control_ANA	 = 1, /*!< GPIO Control Analogy  */
}GPIOControl_TypeDef;
#define IS_GPIO_Control(Ctrl) (((Ctrl) == GPIO_Control_ANA)  || ((Ctrl) == GPIO_Control_DIG) )? 1:0

typedef enum
{ 
	GPIO_Mode_OUT  = 0, /*!< GPIO Output Mode */
	GPIO_Mode_IN	 = 1, /*!< GPIO Input Mode */
}GPIOMode_TypeDef;
#define IS_GPIO_MODE(MODE) (((MODE) == GPIO_Mode_IN)  || ((MODE) == GPIO_Mode_OUT) )? 1:0

typedef enum
{ 
	GPIO_OType_PP = 0,//普通输出
	GPIO_OType_OD = 1 //开漏输出
}GPIOOType_TypeDef;
#define IS_GPIO_OTYPE(OTYPE) (((OTYPE) == GPIO_OType_PP) || ((OTYPE) == GPIO_OType_OD))? 1:0
typedef enum
{ 
	GPIO_PuPd_NOPULL = 0x00,
	GPIO_PuPd_UP     = 0x01,
	GPIO_PuPd_DOWN   = 0x02
}GPIOPuPd_TypeDef;
#define IS_GPIO_PUPD(PUPD) (((PUPD) == GPIO_PuPd_NOPULL) || ((PUPD) == GPIO_PuPd_UP) || \
                            ((PUPD) == GPIO_PuPd_DOWN))? 1:0
typedef enum
{ 
	GPIO_Level_LOW	= 0x00,
	GPIO_Level_HIGH = 0x01,
}GPIOInit_Level;



typedef enum
{ 
	GPIO_PORTA = 0x00,
	GPIO_PORTB = 0x01,		
	GPIO_PORTC = 0x02,
	GPIO_PORTD = 0x03,
	GPIO_PORTE = 0x04,
	GPIO_PORTH = 0x08,

}GPIO_Port_t;

#define IS_GPIO_ALL_PERIPH(PERIPH) (((PERIPH) == GPIO_PORTA) || \
									((PERIPH) == GPIO_PORTB) || \
									((PERIPH) == GPIO_PORTC) || \
									((PERIPH) == GPIO_PORTD)|| \
									((PERIPH) == GPIO_PORTE)|| \
									((PERIPH) == GPIO_PORTH))? 1:0

/** 
  * @brief  GPIO Bit SET and Bit RESET enumeration 
  */ 
typedef enum
{ 
  Bit_RESET = 0,
  Bit_SET
}BitAction;
#define IS_GPIO_BIT_ACTION(ACTION) (((ACTION) == Bit_RESET) || ((ACTION) == Bit_SET))



#define GPIO_Pin_0                 ((uint16_t)0x0001)  /* Pin 0 selected */
#define GPIO_Pin_1                 ((uint16_t)0x0002)  /* Pin 1 selected */
#define GPIO_Pin_2                 ((uint16_t)0x0004)  /* Pin 2 selected */
#define GPIO_Pin_3                 ((uint16_t)0x0008)  /* Pin 3 selected */
#define GPIO_Pin_4                 ((uint16_t)0x0010)  /* Pin 4 selected */
#define GPIO_Pin_5                 ((uint16_t)0x0020)  /* Pin 5 selected */
#define GPIO_Pin_6                 ((uint16_t)0x0040)  /* Pin 6 selected */
#define GPIO_Pin_7                 ((uint16_t)0x0080)  /* Pin 7 selected */
#define GPIO_Pin_8                 ((uint16_t)0x0100)  /* Pin 8 selected */
#define GPIO_Pin_9                 ((uint16_t)0x0200)  /* Pin 9 selected */
#define GPIO_Pin_10                ((uint16_t)0x0400)  /* Pin 10 selected */
#define GPIO_Pin_11                ((uint16_t)0x0800)  /* Pin 11 selected */
#define GPIO_Pin_12                ((uint16_t)0x1000)  /* Pin 12 selected */
#define GPIO_Pin_13                ((uint16_t)0x2000)  /* Pin 13 selected */
#define GPIO_Pin_14                ((uint16_t)0x4000)  /* Pin 14 selected */
#define GPIO_Pin_15                ((uint16_t)0x8000)  /* Pin 15 selected */



#define GPIO_PIN_MASK              ((uint16_t)0xFFFF) /* PIN mask for assert test */
#define IS_GPIO_PIN(PIN)           (((PIN) & GPIO_PIN_MASK ) != (uint16_t)0x00) ? 1:0

#define IS_GET_GPIO_PIN(PIN) (((PIN) == GPIO_Pin_0) || \
                              ((PIN) == GPIO_Pin_1) || \
                              ((PIN) == GPIO_Pin_2) || \
                              ((PIN) == GPIO_Pin_3) || \
                              ((PIN) == GPIO_Pin_4) || \
                              ((PIN) == GPIO_Pin_5) || \
                              ((PIN) == GPIO_Pin_6) || \
															((PIN) == GPIO_Pin_7) || \
															((PIN) == GPIO_Pin_8) || \
															((PIN) == GPIO_Pin_9) || \
															((PIN) == GPIO_Pin_10) || \
															((PIN) == GPIO_Pin_11) || \
															((PIN) == GPIO_Pin_12) || \
															((PIN) == GPIO_Pin_13) || \
															((PIN) == GPIO_Pin_14) || \
                              ((PIN) == GPIO_Pin_15))? 1:0
															

typedef enum
{
	GROUP_AF_ODEFAULT = 0,	//DEFAULT IO function

	GROUP_AF_PGAIN00		= 0x00,		//PA00
	GROUP_AF_PGAIN10 		= 0x00,		//PA03
	GROUP_AF_PGAIN11 		= 0x00,		//PA04	
	GROUP_AF_PGAIN12 		= 0x00,		//PA05	
	GROUP_AF_PGAGND00 	= 0x00,		//PA01
	GROUP_AF_PGAGND10 	= 0x00,		//PA02
	GROUP_AF_PGAGND11 	= 0x00,		//PA07	
	GROUP_AF_PGAGND12 	= 0x00,		//PA06	
	
	GROUP_AF_VCIN00 		= 0x00,		//PA00	
	GROUP_AF_VCIN02	 		= 0x00,		//PA04	
	GROUP_AF_VCIN10	 		= 0x00,		//PA03
	GROUP_AF_VCIN12	 		= 0x00,		//PA05	
	GROUP_AF_VCIN20	 		= 0x00,		//PA04
	GROUP_AF_VCIN22 		= 0x00,		//PA00		
	GROUP_AF_VCIN30	 		= 0x00,		//PA05	
	GROUP_AF_VCIN32	 		= 0x00,		//PA03

	GROUP_AF_INTP0		 	= 0x00,		//PA08
	GROUP_AF_INTP1		 	= 0x00,		//PA09
	GROUP_AF_INTP2		 	= 0x00,		//PA10
	GROUP_AF_INTP3 		 	= 0x00,		//PD03		
	GROUP_AF_INTP4  	 	= 0x00,		//PC10
	GROUP_AF_INTP5			= 0x00,		//PE09
	GROUP_AF_INTP6		 	= 0x00,		//PB00
	GROUP_AF_INTP7		 	= 0x00,		//PB01

	GROUP_AF_CTxD0 		 	= 0x00,		//PC12
	GROUP_AF_CRxD0 		 	= 0x00,		//PC13	
	GROUP_AF_CTxD1		 	= 0x00,		//PA09	
	GROUP_AF_CRxD1		 	= 0x00,		//PA10

	GROUP_AF_VCOUT0		 	= 0x00,		//PB02
	GROUP_AF_VCOUT1		 	= 0x00,		//PB03	
	GROUP_AF_VCOUT2		 	= 0x00,		//PB04
	GROUP_AF_VCOUT3		 	= 0x00,		//PB01
	
	GROUP_AF_TI00   		= 0x03,	 	//PD00 PD07
	GROUP_AF_TI01PD12		= 0x04,		//PD12	
	GROUP_AF_TI01PE12   = 0x03,		//PE12	
	GROUP_AF_TI02		  	= 0x00,		//PB02
	GROUP_AF_TI03 		 	= 0x00,		//PD10
	GROUP_AF_TI04 			= 0x03,		//PB05 PB14	
	GROUP_AF_TI05				= 0x00,		//PE12
	GROUP_AF_TI06				= 0x03,		//PB01
	GROUP_AF_TI06PD06		= 0x04,		//PD07	
	GROUP_AF_TI07 		 	= 0x00,		//PC15	
	
	GROUP_AF_ANO0 		 	= 0x00,		//PB02
	GROUP_AF_ANO1 		 	= 0x00,		//PB03
	
	GROUP_AF_VREF0 		 	= 0x00,		//PB08
	GROUP_AF_VREF1		 	= 0x00,		//PB05

	GROUP_AF_TO00 			= 0x03,	//PD06 PD11 PE10
	GROUP_AF_TO01 		 	= 0x00,		//PD09
	GROUP_AF_TO02 			= 0x03,	//PB03 PC12 PD15	
	GROUP_AF_TO03 		 	= 0x00,		//PB06
	GROUP_AF_TO04 			= 0x03,	//PB04 PB13 PC13 PD12	
	GROUP_AF_TO05				= 0x00,		//PE13	
	GROUP_AF_TO06PB02		= 0x03,	  //PB02	
	GROUP_AF_TO06PD14		= 0x04,	//PD14	
	GROUP_AF_TO07 		 	= 0x00,		//PC14	
	
	GROUP_AF_SS00 			= 0x05,	//PD09 PE06	
	GROUP_AF_SS10				= 0x05,	//PD03 PE13	
	GROUP_AF_SS20	 	    = 0x00,		//PC14	
	GROUP_AF_SS30 		 	= 0x00,		//PB07


	GROUP_AF_SDAA1 		 	= 0x00,		//PB10
	GROUP_AF_SCLA1 		 	= 0x00,		//PB11
	GROUP_AF_SCLK30		 	= 0x00,		//PB13
	GROUP_AF_SCL30 		 	= 0x00,		//PB13
	GROUP_AF_TxD3 		 	= 0x00,		//PB14
	GROUP_AF_SDO30 		 	= 0x00,		//PB14
	GROUP_AF_RxD3 		 	= 0x00,		//PB15
	GROUP_AF_SDI30 		 	= 0x00,		//PB15
	GROUP_AF_SDA30 		 	= 0x00,		//PB15
	
	GROUP_AF_DBD0 		 	= 0x00,		//PC00
	GROUP_AF_GTIOC15B	 	= 0x00,		//PC00
	
	GROUP_AF_DBD1 		 	= 0x00,		//PC01
	GROUP_AF_GTIOC15A	 	= 0x00,		//PC01
	
	GROUP_AF_DBD2 		 	= 0x00,		//PC02
	GROUP_AF_GTIOC14B	 	= 0x00,		//PC02
	
	GROUP_AF_DBD3 		 	= 0x00,		//PC03
	GROUP_AF_GTIOC14A	 	= 0x00,		//PC03
	
	GROUP_AF_DBD4 		 	= 0x00,		//PC04
	GROUP_AF_GTIOC12B	 	= 0x00,		//PC04
	
	GROUP_AF_DBD5 		 	= 0x00,		//PC05
	GROUP_AF_GTIOC12A	 	= 0x00,		//PC05
	
	GROUP_AF_DBD6 		 	= 0x00,		//PC06
	GROUP_AF_GTIOC11B	 	= 0x00,		//PC06
	
	GROUP_AF_DBD7 		 	= 0x00,		//PC07
	GROUP_AF_GTIOC11A	 	= 0x00,		//PC07
	
	GROUP_AF_DBD8 		 	= 0x00,		//PC08
	GROUP_AF_GTIOC10B	 	= 0x00,		//PC08
	
	GROUP_AF_DBD9 		 	= 0x00,		//PC09
	GROUP_AF_GTIOC10A	 	= 0x00,		//PC09
	
	GROUP_AF_DBD10 		 	= 0x00,		//PC10
	
	GROUP_AF_DBD14 		 	= 0x00,		//PC11
	
	GROUP_AF_DBD11 		 	= 0x00,		//PC12
	GROUP_AF_GTIOC13B	 	= 0x00,		//PC12
	
	GROUP_AF_DBD12 		 	= 0x00,		//PC13
	GROUP_AF_GTIOC13A	 	= 0x00,		//PC13
	
	GROUP_AF_DBD13 		 	= 0x00,		//PC14
	
	GROUP_AF_DBD15 		 	= 0x00,		//PC15
	GROUP_AF_SDO31 	    = 0x00,		//PC15

	GROUP_AF_SCLA0 		 	= 0x00,		//PD00
	GROUP_AF_TI12 	    = 0x00,		//PD00
	GROUP_AF_TO12 		 	= 0x00,		//PD00
	
	GROUP_AF_SDAA0 		 	= 0x00,		//PD01

	GROUP_AF_QSSL		    = 0x00,		//PD05
	
	GROUP_AF_QIO0 		 	= 0x00,		//PD06
	GROUP_AF_TxD0 		 	= 0x00,		//PD06
	GROUP_AF_SDO00 		 	= 0x00,		//PD06
	
	GROUP_AF_SCLK00		 	= 0x00,		//PD07
	GROUP_AF_SCL00 		 	= 0x00,		//PD07
	GROUP_AF_QIO1 		 	= 0x00,		//PD07
	
	GROUP_AF_RxD0		 		= 0x00,		//PD08
	GROUP_AF_SDI00 		 	= 0x00,		//PD08
	GROUP_AF_QIO2 		 	= 0x00,		//PD08
	GROUP_AF_SDA00		 	= 0x00,		//PD08
	
	GROUP_AF_SPIHS0NSS 	= 0x00,		//PD09
	GROUP_AF_QIO3 		 	= 0x00,		//PD09
	
	GROUP_AF_SCLK31		 	= 0x00,		//PD10
	GROUP_AF_SCL31 		 	= 0x00,		//PD10

	GROUP_AF_TI13 		 	= 0x00,		//PD11
	GROUP_AF_SDI31 		 	= 0x00,		//PD11
	GROUP_AF_TO13 		 	= 0x00,		//PD11
	GROUP_AF_SDA31		 	= 0x00,		//PD11
	
	GROUP_AF_SPIHS0MOSI	= 0x00,		//PD12
	GROUP_AF_SPIHS0SCK	= 0x00,		//PD14
	GROUP_AF_SPIHS0MISO	= 0x00,		//PD15
	
	GROUP_AF_QSPCLK			= 0x00,		//PE00
	GROUP_AF_GTIOC16B		= 0x00,		//PE07
	GROUP_AF_DBWRB  		= 0x00,		//PE07

	GROUP_AF_GTIOC16A		= 0x00,		//PE08
	GROUP_AF_DBRDB  		= 0x00,		//PE08
	
	GROUP_AF_DBA0  			= 0x00,		//PE09
	
	GROUP_AF_GTIOC17B		= 0x00,		//PE11
	
//KEY
	GROUP_AF_KR0 = 0x00,		//PA00
	GROUP_AF_KR1 = 0x00,		//PA01
	GROUP_AF_KR2 = 0x00,		//PA02
	GROUP_AF_KR3 = 0x00,		//PA11
	GROUP_AF_KR4 = 0x00,		//PA03
	GROUP_AF_KR5 = 0x00,		//PA04
	GROUP_AF_KR6 = 0x00,		//PA05
	GROUP_AF_KR7 = 0x00,		//PA12

//BUZ
	GROUP_AF_PCLBUZ0 = 0x03,	//PE05
	GROUP_AF_PCLBUZ1 = 0x00,	//PD03

//PB00
  GROUP_AF_TI10 		= 0x01,	//PB00 PD02
	GROUP_AF_TO10 		= 0x01,	//PB00 PD02
	GROUP_AF_GTIOC00A = 0x02,	//PB00 PD02 PD12
	GROUP_AF_GTETRGC  = 0x07,	//PB00 PB06 PC10 PD06 PH04

	GROUP_AF_TI11 			= 0x01,	//PB01 PD01
	GROUP_AF_TO11 			= 0x01,	//PB01 PD01
	GROUP_AF_GTIOC00B 	= 0x02,	//PB01 PD01 PD15

	GROUP_AF_SPIHS1NSS 	= 0x06,	//PB01 PC11
	GROUP_AF_GTETRGD  	= 0x07,	//PB01 PB07 PD03 PE09 PE13 PH03


	GROUP_AF_SPIHS1SCK 	= 0x06,	//PB02 PC15 PD02
	GROUP_AF_GPTU  			= 0x07,	//PB02 PB13


	GROUP_AF_SPIHS1MOSI = 0x06,	//PB03 PD11
	GROUP_AF_GPTV  			= 0x07,	//PB03 PB14


	GROUP_AF_ADTRG2			= 0x04,	//PB04 PD11
	GROUP_AF_SPIHS1MISO = 0x06,	//PB04 PD10
	GROUP_AF_GPTW  			= 0x07,	//PB04 PB15
	
	GROUP_AF_TI15 			= 0x01,	//PB05 PC15 PE00
	GROUP_AF_TO15 			= 0x01,	//PB05 PC15 PE00
	GROUP_AF_GTIOC03A		= 0x02,	//PB05 PC11 PE00

	GROUP_AF_ADTRG1			= 0x04,	//PB05 PD10
	GROUP_AF_GTETRGA		= 0x07,	//PB05 PD08 PE01 PE10

	GROUP_AF_GTIOC17A		= 0x02,	//PB06 PE10

	GROUP_AF_TI16 			= 0x01,	//PB07 PC11 PE12
	GROUP_AF_TO16 			= 0x01,	//PB07 PC11 PE12
	GROUP_AF_GTIOC02B		= 0x02,	//PB07 PC15 PD04 PE12

	GROUP_AF_TI14 			= 0x01,	//PB08 PD09 PD10
	GROUP_AF_TO14 			= 0x01,	//PB08 PD09 PD10
	GROUP_AF_GTIOC03B		= 0x02,	//PB08 PC14 PD09
	GROUP_AF_ADTRG0			= 0x04,	//PB08 PC13 PC15
	GROUP_AF_GTETRGB		= 0x07,	//PB08 PD07 PE02 PE11
	 
	GROUP_AF_SDO21PB13	= 0x06,	//PB13

	GROUP_AF_SDI21PB14	= 0x06,	//PB14
	GROUP_AF_SDA21PB14	= 0x06,	//PB14

	GROUP_AF_SCL21PB15	= 0x06,	//PB15
	GROUP_AF_SCLK21PB15	= 0x06,	//PB15	

	GROUP_AF_GTOVLO 		= 0x07,	//PC00 PC06 PD13
	GROUP_AF_GTOVUP 		= 0x07,	//PC01 PC07 PD14
	GROUP_AF_GTOULO 		= 0x07,	//PC02 PC08 PD15
	GROUP_AF_GTOUUP 		= 0x07,	//PC03 PC09 PD12
	GROUP_AF_GTOWLO 		= 0x07,	//PC04 PD04 PE08
	GROUP_AF_GTOWUP 		= 0x07,	//PC05 PD05 PE07
	
	GROUP_AF_TI17PC14		= 0x01,	//PC14 
	GROUP_AF_TO17PC14		= 0x01,	//PC14

	GROUP_AF_SCLK20			= 0x05,	//PC15  PD02
	GROUP_AF_SCL20     	= 0x05,	//PC15  PD02 
	
	GROUP_AF_GTIOC01A		= 0x02,	//PD00 PD14

	GROUP_AF_SCL21PD00	= 0x04,	//PD00 
	GROUP_AF_SCLK21PD00	= 0x04,	//PD00 
	GROUP_AF_RxD2     	= 0x05,	//PD00 PD10
	GROUP_AF_SDI20     	= 0x05,	//PD00 PD10
	GROUP_AF_SDA20     	= 0x05,	//PD00 PD10
	
	GROUP_AF_SDI21PD01 	= 0x04,	//PD01 
	GROUP_AF_SDA21PD01  = 0x04,	//PD01 
	GROUP_AF_TxD2 			= 0x05,	//PD01 PD11
	GROUP_AF_SDO20 			= 0x05,	//PD01 PD11
	
	GROUP_AF_SDO21PD02	= 0x04,	//PD02

	GROUP_AF_SDO11			= 0x06,	//PD03 PD09
	
	
	GROUP_AF_TxD1 			= 0x05,	//PD04 PE00
	GROUP_AF_SDO10 			= 0x05,	//PD04 PE00
	GROUP_AF_SDI11 			= 0x06,	//PD04 PE00
	GROUP_AF_SDA11 			= 0x06,	//PD04 PE00
	
	GROUP_AF_GTIOC02A		= 0x02,	//PD05 PD10 PE06 PE13
	GROUP_AF_RxD1 			= 0x05,	//PD05 PE12
	GROUP_AF_SDI10 			= 0x05,	//PD05 PE12
	GROUP_AF_SDA10 			= 0x05,	//PD05 PE12
	GROUP_AF_SCL11 			= 0x06,	//PD05 PE12
	GROUP_AF_SCLK11			= 0x06,	//PD05 PE12
	

	GROUP_AF_SDI01 			= 0x06,	//PD06
	GROUP_AF_SDA01 			= 0x06,	//PD06
	

	GROUP_AF_SDO01 			= 0x06,	//PD07
	
	GROUP_AF_SCL01 			= 0x06,	//PD08
	GROUP_AF_SCLK01			= 0x06,	//PD08
	
	GROUP_AF_ADST0			= 0x04,	//PD09 PE06

	
	GROUP_AF_GTIOC01B 	= 0x02,	//PD11 PD13
	

	GROUP_AF_SCL10 			= 0x05,	//PD12 PD13
	GROUP_AF_SCLK10			= 0x05,	//PD12 PD13
	

	
	GROUP_AF_GPIO     	= 0x01,	//PE01 PE02 PE03 PE05 PE06
	GROUP_AF_RTC1HZ 		= 0x03,	//PE01
	
	GROUP_AF_ADST2 		  = 0x04,	//PE02
	
	GROUP_AF_TI17PE06		= 0x03,	//PE06
	GROUP_AF_TO17PE06		= 0x03,	//PE06
	

	
	GROUP_AF_TI17PE13		= 0x01,	//PE13 
	GROUP_AF_TO17PE13		= 0x01,	//PE13
	
}GROUP_AF_t;
	
	
#define IS_GPIO_AF(GPIO_AF) (((GPIO_AF) == GROUP_AF_ODEFAULT  || (GPIO_AF) == GROUP_AF_VCIN22 ||\
						(GPIO_AF) == GROUP_AF_PGAIN00 || (GPIO_AF) == GROUP_AF_VCIN00 || (GPIO_AF) == GROUP_AF_PGAGND00 || (GPIO_AF) == GROUP_AF_PGAGND10 || (GPIO_AF) == GROUP_AF_VCIN10 || \
						(GPIO_AF) == GROUP_AF_VCIN32 || (GPIO_AF) == GROUP_AF_PGAIN10 || (GPIO_AF) == GROUP_AF_VCIN02 || (GPIO_AF) == GROUP_AF_VCIN20 || (GPIO_AF) == GROUP_AF_PGAIN11 || \
						(GPIO_AF) == GROUP_AF_VCIN12 || (GPIO_AF) == GROUP_AF_VCIN30 || (GPIO_AF) == GROUP_AF_PGAIN12 || (GPIO_AF) == GROUP_AF_PGAGND12 || (GPIO_AF) == GROUP_AF_PGAGND11 || \
						(GPIO_AF) == GROUP_AF_INTP0 || (GPIO_AF) == GROUP_AF_INTP1 || (GPIO_AF) == GROUP_AF_CTxD1 || (GPIO_AF) == GROUP_AF_INTP2 || (GPIO_AF) == GROUP_AF_CRxD1 || \
						(GPIO_AF) == GROUP_AF_INTP6 || (GPIO_AF) == GROUP_AF_INTP7 || (GPIO_AF) == GROUP_AF_VCOUT3 || (GPIO_AF) == GROUP_AF_VCOUT0 || (GPIO_AF) == GROUP_AF_TI02 || \
						(GPIO_AF) == GROUP_AF_ANO0 || (GPIO_AF) == GROUP_AF_VCOUT1 || (GPIO_AF) == GROUP_AF_ANO1 || (GPIO_AF) == GROUP_AF_VCOUT2 || (GPIO_AF) == GROUP_AF_VREF1 || \
						(GPIO_AF) == GROUP_AF_TO03 || (GPIO_AF) == GROUP_AF_SS30 || (GPIO_AF) == GROUP_AF_VREF0 || (GPIO_AF) == GROUP_AF_SDAA1 || (GPIO_AF) == GROUP_AF_SCLA1 || \
						(GPIO_AF) == GROUP_AF_SCLK30 || (GPIO_AF) == GROUP_AF_SCL30 || (GPIO_AF) == GROUP_AF_TxD3 || (GPIO_AF) == GROUP_AF_SDO30 || (GPIO_AF) == GROUP_AF_RxD3 || \
						(GPIO_AF) == GROUP_AF_SDI30 || (GPIO_AF) == GROUP_AF_SDA30 || (GPIO_AF) == GROUP_AF_DBD0 || (GPIO_AF) == GROUP_AF_GTIOC15B || (GPIO_AF) == GROUP_AF_DBD1 || \
						(GPIO_AF) == GROUP_AF_GTIOC15A || (GPIO_AF) == GROUP_AF_DBD2 || (GPIO_AF) == GROUP_AF_GTIOC14B || (GPIO_AF) == GROUP_AF_DBD3 || (GPIO_AF) == GROUP_AF_GTIOC14A || \
						(GPIO_AF) == GROUP_AF_DBD4 || (GPIO_AF) == GROUP_AF_GTIOC12B || (GPIO_AF) == GROUP_AF_DBD5 || (GPIO_AF) == GROUP_AF_GTIOC12A || (GPIO_AF) == GROUP_AF_DBD6 || \
						(GPIO_AF) == GROUP_AF_GTIOC11B || (GPIO_AF) == GROUP_AF_DBD7 || (GPIO_AF) == GROUP_AF_GTIOC11A || (GPIO_AF) == GROUP_AF_DBD8 || (GPIO_AF) == GROUP_AF_GTIOC10B || \
						(GPIO_AF) == GROUP_AF_DBD9 || (GPIO_AF) == GROUP_AF_GTIOC10A || (GPIO_AF) == GROUP_AF_DBD10 || (GPIO_AF) == GROUP_AF_INTP4 || (GPIO_AF) == GROUP_AF_DBD14 || \
						(GPIO_AF) == GROUP_AF_DBD11 || (GPIO_AF) == GROUP_AF_GTIOC13B || (GPIO_AF) == GROUP_AF_CTxD0 || (GPIO_AF) == GROUP_AF_DBD12 || (GPIO_AF) == GROUP_AF_GTIOC13A || \
						(GPIO_AF) == GROUP_AF_CRxD0 || (GPIO_AF) ==  GROUP_AF_DBD13 || (GPIO_AF) == GROUP_AF_SS20 || (GPIO_AF) == GROUP_AF_TO07 || (GPIO_AF) == GROUP_AF_DBD15 || \
						(GPIO_AF) == GROUP_AF_SDO31 || (GPIO_AF) == GROUP_AF_TI07 || (GPIO_AF) == GROUP_AF_SCLA0 || (GPIO_AF) == GROUP_AF_TI12 || (GPIO_AF) == GROUP_AF_TO12 || \
						(GPIO_AF) == GROUP_AF_SDAA0 || (GPIO_AF) == GROUP_AF_INTP3 || (GPIO_AF) == GROUP_AF_QSSL || (GPIO_AF) == GROUP_AF_QIO0 || (GPIO_AF) == GROUP_AF_TxD0 || \
						(GPIO_AF) == GROUP_AF_SDO00 || (GPIO_AF) == GROUP_AF_SCLK00 || (GPIO_AF) == GROUP_AF_SCL00 || (GPIO_AF) == GROUP_AF_QIO1 || (GPIO_AF) == GROUP_AF_RxD0 || \
						(GPIO_AF) == GROUP_AF_SDI00 || (GPIO_AF) == GROUP_AF_QIO2 || (GPIO_AF) == GROUP_AF_SDA00 || (GPIO_AF) == GROUP_AF_SPIHS0NSS || (GPIO_AF) == GROUP_AF_TO01 || \
						(GPIO_AF) == GROUP_AF_QIO3 || (GPIO_AF) == GROUP_AF_SCLK31 || (GPIO_AF) == GROUP_AF_SCL31 || (GPIO_AF) == GROUP_AF_TI03 || (GPIO_AF) == GROUP_AF_TI13 || \
						(GPIO_AF) == GROUP_AF_SDI31 || (GPIO_AF) == GROUP_AF_TO13 || (GPIO_AF) == GROUP_AF_SDA31 || (GPIO_AF) == GROUP_AF_SPIHS0MOSI || (GPIO_AF) == GROUP_AF_SPIHS0SCK || \
						(GPIO_AF) == GROUP_AF_SPIHS0MISO || (GPIO_AF) == GROUP_AF_QSPCLK || (GPIO_AF) == GROUP_AF_GTIOC16B || (GPIO_AF) == GROUP_AF_DBWRB || (GPIO_AF) == GROUP_AF_GTIOC16A || \
						(GPIO_AF) == GROUP_AF_DBRDB || (GPIO_AF) == GROUP_AF_INTP5 || (GPIO_AF) == GROUP_AF_DBA0 || (GPIO_AF) == GROUP_AF_GTIOC17B || (GPIO_AF) == GROUP_AF_TI05 || \
						(GPIO_AF) == GROUP_AF_TO05 || (GPIO_AF) == GROUP_AF_KR0 || (GPIO_AF) == GROUP_AF_KR1 || (GPIO_AF) == GROUP_AF_KR2 || (GPIO_AF) == GROUP_AF_KR3 || \
						(GPIO_AF) == GROUP_AF_KR4 || (GPIO_AF) == GROUP_AF_KR5 || (GPIO_AF) == GROUP_AF_KR6 || (GPIO_AF) == GROUP_AF_KR7 || (GPIO_AF) == GROUP_AF_TI10 || \
						(GPIO_AF) == GROUP_AF_TO10 || (GPIO_AF) == GROUP_AF_GTIOC00A || (GPIO_AF) == GROUP_AF_GTETRGC || (GPIO_AF) == GROUP_AF_TI11 || (GPIO_AF) == GROUP_AF_TO11 || \
						(GPIO_AF) == GROUP_AF_GTIOC00B || (GPIO_AF) == GROUP_AF_TI06 || (GPIO_AF) == GROUP_AF_SPIHS1NSS || (GPIO_AF) == GROUP_AF_GTETRGD || (GPIO_AF) == GROUP_AF_TO06PB02 || \
						(GPIO_AF) == GROUP_AF_SPIHS1SCK || (GPIO_AF) == GROUP_AF_GPTU || (GPIO_AF) == GROUP_AF_TO02 || (GPIO_AF) == GROUP_AF_SPIHS1MOSI || (GPIO_AF) == GROUP_AF_GPTV || \
						(GPIO_AF) == GROUP_AF_TO04 || (GPIO_AF) == GROUP_AF_ADTRG2 || (GPIO_AF) == GROUP_AF_SPIHS1MISO || (GPIO_AF) == GROUP_AF_GPTW || (GPIO_AF) == GROUP_AF_TI15 || \
						(GPIO_AF) == GROUP_AF_TO15 || (GPIO_AF) == GROUP_AF_GTIOC03A || (GPIO_AF) == GROUP_AF_TI04 || (GPIO_AF) ==GROUP_AF_ADTRG1  || (GPIO_AF) == GROUP_AF_GTETRGA || \
						(GPIO_AF) == GROUP_AF_GTIOC17A || (GPIO_AF) ==  GROUP_AF_TI16|| (GPIO_AF) == GROUP_AF_TO16 || (GPIO_AF) == GROUP_AF_GTIOC02B || (GPIO_AF) == GROUP_AF_TI14 || \
						(GPIO_AF) == GROUP_AF_TO14 || (GPIO_AF) == GROUP_AF_GTIOC03B || (GPIO_AF) == GROUP_AF_ADTRG0 || (GPIO_AF) == GROUP_AF_GTETRGB || (GPIO_AF) == GROUP_AF_SDO21PB13 || \
						(GPIO_AF) == GROUP_AF_SDI21PB14 || (GPIO_AF) == GROUP_AF_SDA21PB14 || (GPIO_AF) == GROUP_AF_SCL21PB15 || (GPIO_AF) == GROUP_AF_SCLK21PB15 || (GPIO_AF) == GROUP_AF_GTOVLO || \
						(GPIO_AF) == GROUP_AF_GTOVUP || (GPIO_AF) == GROUP_AF_GTOULO || (GPIO_AF) == GROUP_AF_GTOUUP || (GPIO_AF) == GROUP_AF_GTOWLO || (GPIO_AF) == GROUP_AF_GTOWUP || \
						(GPIO_AF) == GROUP_AF_TI17PC14 || (GPIO_AF) == GROUP_AF_TO17PC14 || (GPIO_AF) == GROUP_AF_SCLK20 || (GPIO_AF) == GROUP_AF_GTIOC01A || (GPIO_AF) == GROUP_AF_TI00 || \
						(GPIO_AF) == GROUP_AF_SCL21PD00 || (GPIO_AF) == GROUP_AF_SCLK21PD00 || (GPIO_AF) == GROUP_AF_RxD2 || (GPIO_AF) == GROUP_AF_SDI20 || (GPIO_AF) == GROUP_AF_SDA20 || \
						(GPIO_AF) == GROUP_AF_SDI21PD01 || (GPIO_AF) == GROUP_AF_SDA21PD01 || (GPIO_AF) == GROUP_AF_TxD2 || (GPIO_AF) == GROUP_AF_SDO20 || (GPIO_AF) == GROUP_AF_SDO21PD02 || \
						(GPIO_AF) == GROUP_AF_SS10 || (GPIO_AF) == GROUP_AF_SDO11 || (GPIO_AF) == GROUP_AF_TxD1 || (GPIO_AF) == GROUP_AF_SDO10 || (GPIO_AF) == GROUP_AF_SDI11 || \
						(GPIO_AF) == GROUP_AF_SDA11 || (GPIO_AF) == GROUP_AF_GTIOC02A || (GPIO_AF) == GROUP_AF_RxD1 || (GPIO_AF) == GROUP_AF_SDI10 || (GPIO_AF) == GROUP_AF_SDA10 || \
						(GPIO_AF) == GROUP_AF_SCL11 || (GPIO_AF) == GROUP_AF_SCLK11 || (GPIO_AF) == GROUP_AF_TO00 || (GPIO_AF) == GROUP_AF_SDI01 || (GPIO_AF) == GROUP_AF_SDA01 || \
						(GPIO_AF) == GROUP_AF_TI06PD06 || (GPIO_AF) == GROUP_AF_SDO01 || (GPIO_AF) == GROUP_AF_SCL01 || (GPIO_AF) == GROUP_AF_SCLK01 || (GPIO_AF) == GROUP_AF_ADST0 ||\
						(GPIO_AF) == GROUP_AF_SCLK10 || (GPIO_AF) == GROUP_AF_SCL10 || (GPIO_AF) == GROUP_AF_TI01PD12 || (GPIO_AF) == GROUP_AF_GTIOC01B || (GPIO_AF) == GROUP_AF_SS00 ||\
						(GPIO_AF) == GROUP_AF_TO06PD14 || (GPIO_AF) == GROUP_AF_GPIO || (GPIO_AF) == GROUP_AF_RTC1HZ || (GPIO_AF) == GROUP_AF_ADST2 || (GPIO_AF) == GROUP_AF_TI17PE06 ||\
						(GPIO_AF) == GROUP_AF_TO17PE06 || (GPIO_AF) == GROUP_AF_TI01PE12 || (GPIO_AF) == GROUP_AF_TI17PE13 || (GPIO_AF) == GROUP_AF_TO17PE13 || (GPIO_AF) == GROUP_AF_PCLBUZ0 ||(GPIO_AF) == GROUP_AF_PCLBUZ1 \
					)? 1:0)



typedef enum
{
	GPIO_PIN_START = 0x00, 
	GPIO_PA00 = 0x01,
	GPIO_PA01 = 0x02,
	GPIO_PA02 = 0x03,
	GPIO_PA03 = 0x04,
	GPIO_PA04 = 0x05,
	GPIO_PA05 = 0x06,
	GPIO_PA06 = 0x07,
	GPIO_PA07 = 0x08,
	GPIO_PA08 = 0x09,
	GPIO_PA09 = 0x0A,
	GPIO_PA10 = 0x0B,
	GPIO_PA11 = 0x0C,
	GPIO_PA12 = 0x0D,
	GPIO_PA13 = 0x0E,
	GPIO_PA14 = 0x0F,
	GPIO_PA15 = 0x10,
	GPIO_PB00 = 0x11,
	GPIO_PB01 = 0x12,
	GPIO_PB02 = 0x13,
	GPIO_PB03 = 0x14,
	GPIO_PB04 = 0x15,
	GPIO_PB05 = 0x16,
	GPIO_PB06 = 0x17,
	GPIO_PB07 = 0x18,
	GPIO_PB08 = 0x19,
	GPIO_PB09 = 0x1A,
	GPIO_PB10 = 0x1B,
	GPIO_PB11 = 0x1C,
	GPIO_PB12 = 0x1D,
	GPIO_PB13 = 0x1E,
	GPIO_PB14 = 0x1F,
	GPIO_PB15 = 0x20,
	GPIO_PC00 = 0x21,
	GPIO_PC01 = 0x22,
	GPIO_PC02 = 0x23,
	GPIO_PC03 = 0x24,
	GPIO_PC04 = 0x25,
	GPIO_PC05 = 0x26,
	GPIO_PC06 = 0x27,
	GPIO_PC07 = 0x28,
	GPIO_PC08 = 0x29,
	GPIO_PC09 = 0x2A,
	GPIO_PC10 = 0x2B,
	GPIO_PC11 = 0x2C,
	GPIO_PC12 = 0x2D,
	GPIO_PC13 = 0x2E,	
	GPIO_PC14 = 0x2F,	
	GPIO_PC15 = 0x30,
	GPIO_PD00 = 0x31,	
	GPIO_PD01 = 0x32,		
	GPIO_PD02 = 0x33,		
	GPIO_PD03 = 0x34,		
	GPIO_PD04 = 0x35,		
	GPIO_PD05 = 0x36,		
	GPIO_PD06 = 0x37,		
	GPIO_PD07 = 0x38,
	GPIO_PD08 = 0x39,
	GPIO_PD09 = 0x3A,
	GPIO_PD10 = 0x3B,
	GPIO_PD11 = 0x3C,
	GPIO_PD12 = 0x3D,
	GPIO_PD13 = 0x3E,
	GPIO_PD14 = 0x3F,
	GPIO_PD15 = 0x40,
	GPIO_PE00 = 0x41,	
	GPIO_PE01 = 0x42,		
	GPIO_PE02 = 0x43,		
	GPIO_PE03 = 0x44,		
	GPIO_PE04 = 0x45,		
	GPIO_PE05 = 0x46,		
	GPIO_PE06 = 0x47,		
	GPIO_PE07 = 0x48,
	GPIO_PE08 = 0x49,
	GPIO_PE09 = 0x4A,
	GPIO_PE10 = 0x4B,
	GPIO_PE11 = 0x4C,
	GPIO_PE12 = 0x4D,
	GPIO_PE13 = 0x4E,
	GPIO_PH00 = 0x4F,
	GPIO_PH01 = 0x50,		
	GPIO_PH02 = 0x51,			
	GPIO_PH03 = 0x52,			
	GPIO_PH04 = 0x53,			
	GPIO_PIN_END = 0x54,
}GPIO_Source_t;

 

typedef struct 
{
	uint16_t GPIO_Pin;						/*!< Specifies the GPIO pins to be configured.
										  This parameter can be any value of @ref GPIO_pins_define */
	GPIOMode_TypeDef GPIO_Mode;	  /*!< Specifies the operating mode for the selected pins.
									   This parameter can be a value of @ref GPIOMode_TypeDef */
	GPIOControl_TypeDef GPIO_Ctrl;    /*!< Specifies the control mode for the selected pins.
									   This parameter can be a value of @ref GPIOControl_TypeDef */
	GPIOOType_TypeDef GPIO_OType;   /*!< Specifies the operating output type for the selected pins.
										This parameter can be a value of @ref GPIOOType_TypeDef */
	GPIOPuPd_TypeDef GPIO_PuPd; 	/*!< Specifies the operating Pull-up/Pull down for the selected pins.
										 This parameter can be a value of @ref GPIOPuPd_TypeDef */
	GPIOInit_Level GPIO_Level;  /*!< Specifies the gpio init level for the selected pins.
										  This parameter can be a value of @ref GPIOInit_Level */
	
}GPIO_InitTypeDef;


/***************************************************************
*  PinCFG: PA00CFG, PA01CFG, ....PH04CFG
*  MuxMode: GROUP_AF_RXD0,GROUP_AF_RXD1, .....,GROUP_AF_PCLBUZ1
******************************************************************/
#define SYS_SET_IOCFG(PinCFG, MuxMode)	PinCFG = MuxMode;


void GPIO_Output_Enable(__IO uint16_t *port, uint16_t pinMsk);
void GPIO_Input_Enable(__IO uint16_t *port, uint16_t pinMsk);
void GPIO_PullUp_Enable(__IO uint16_t *port, uint16_t pinMsk);
void GPIO_PullUp_Disable(__IO uint16_t *port, uint16_t pinMsk);
void GPIO_PullDown_Enable(__IO uint16_t *port, uint16_t pinMsk);
void GPIO_PullDown_Disable(__IO uint16_t *port, uint16_t pinMsk);
void GPIO_Set_Value(__IO uint16_t *port, uint16_t value);
//uint8_t GPIO_Get_Value(__IO uint8_t *port);

uint8_t GPIO_ReadInputDataBit(GPIO_Port_t PORTx, uint16_t GPIO_Pin);
uint8_t GPIO_ReadOutputDataBit(GPIO_Port_t PORTx, uint16_t GPIO_Pin);
void GPIO_SetBits(GPIO_Port_t  PORTx, uint16_t GPIO_Pin);
void GPIO_ResetBits(GPIO_Port_t PORTx, uint16_t GPIO_Pin);
void GPIO_Init(GPIO_Port_t PORTx,GPIO_InitTypeDef* GPIO_InitStruct);
int GPIO_PinAFConfig(GPIO_Port_t PORTx, uint16_t GPIO_Pin, GROUP_AF_t GPIO_AF);
//void GPIO_SPI_GroupConfig(uint8_t groupNum);
#endif

