#ifndef __PGA_H__
#define __PGA_H__
#include <stdio.h>
#include <stdint.h>
#include "common.h"

typedef enum 
{    
	PGA00 = 0,  // PGA00    
	PGA10 = 1,  // PGA10    
	PGA11 = 2,  // PGA11 
	PGA12 = 3,  // PGA12 
} PGA_Channel_t;

typedef enum 
{    
  PGA_FullDif     = 0,  // PGA work as full difference 
	PGA_ShamDif     = 1,	// PGA work as sham difference 	
} PGA_Mod_t;

typedef enum 
{    
  PGA_Out_notADC        = 0,  // PGA out not to ADC
	PGA_Out_DirectADC     = 1,	// PGA input direct to ADC
	PGA_Out_AmpADC        = 2,	// PGA amplifer to ADC	
	
} PGA_Out_t;

typedef enum 
{    
  PGA_FDGain_5  = 0, // full difference Gain Amplifier:*5 
	PGA_FDGain_8  = 1, // full difference Gain Amplifier:*8	 
	PGA_FDGain_10	= 2, // full difference Gain Amplifier:*10 
	PGA_FDGain_16	= 3, // full difference Gain Amplifier:*16 

} PGA_FDifGain_t;

typedef enum 
{    
  PGA_SDGain_2  = 0, // ShamDifference Gain Amplifier:*2
	PGA_SDGain_2p5  = 1, // ShamDifference Gain Amplifier:*2.5
	PGA_SDGain_2p667  = 2, // ShamDifference Gain Amplifier:*2.667
	PGA_SDGain_2p857  = 3, // ShamDifference Gain Amplifier:*2.857
	PGA_SDGain_3p077  = 4, // ShamDifference Gain Amplifier:*3.077
	PGA_SDGain_3p333  = 5, // ShamDifference Gain Amplifier:*3.333
	PGA_SDGain_3p636  = 6, // ShamDifference Gain Amplifier:*3.636
	PGA_SDGain_4  = 7, // ShamDifference Gain Amplifier:*4
	PGA_SDGain_4p444  = 8, // ShamDifference Gain Amplifier:*4.444
	PGA_SDGain_5  = 9, // ShamDifference Gain Amplifier:*5
	PGA_SDGain_5p714  = 10, // ShamDifference Gain Amplifier:*5.714
	PGA_SDGain_6p667  = 11, // ShamDifference Gain Amplifier:*6.667
	PGA_SDGain_8  = 12, // ShamDifference Gain Amplifier:*8
	PGA_SDGain_10  = 13, // ShamDifference Gain Amplifier:*10
	PGA_SDGain_16  = 14, // ShamDifference Gain Amplifier:*16
	PGA_SDGain_1  = 15, // ShamDifference Gain Amplifier:*1
	
} PGA_SDifGain_t;

typedef enum 
{    
  PGA_GND_Inter     = 0,  // Select internal GND as PGA GND 
	PGA_GND_PGAGND    = 1,	// Select PGAGND as PGA+ input	
} PGA_Gnd_t;


typedef struct
{
	PGA_Channel_t PGA_Channel;	/*!< Specifies the channel of pga.*/
										   	
	PGA_Mod_t PGA_Mod;/*!< Specifies the mode 
								 This parameter can be a value of @ref PGA_Mod_t */
	uint8_t PGA_Gain;/*!< Specifies the Programmable gain amplifier amplification factor selection 
								 This parameter can be a value of @ref PGA_FDifGain_t or PGA_SDifGain_t*/
//	PGA_SDifGain_t 	PGA_SDGain;	/*!< Specifies the Programmable gain amplifier amplification factor selection 
//								 This parameter can be a value of @ref PGA_SDifGain_t */	
  PGA_Gnd_t PGA_Gnd;	/*!< Specifies the Programmable gain gnd 
								 This parameter can be a value of @ref PGA_Gnd_t */	
	PGA_Out_t PGA_Out;  /*!< Specifies the Programmable gain out mode  
								 This parameter can be a value of @ref PGA_Out_t */	
}PGA_InitTypeDef;


void PGA_Init(PGA_InitTypeDef *PGA_InitStruct);
void PGA_Start(PGA_Channel_t ch);
void PGA_Stop(PGA_Channel_t ch);

#endif
