#ifndef __QSPI_H__
#define __QSPI_H__

#include "common.h"


/**************************************************************************
* QSPI Registers defination
**************************************************************************/
#define SFMSMD  (*((volatile uint32_t *)0x64000000))
#define SFMSSC  (*((volatile uint32_t *)0x64000004))
#define SFMSKC  (*((volatile uint32_t *)0x64000008))
#define SFMSST  (*((volatile uint32_t *)0x6400000C))
//#define SFMCOM  (*((volatile uint32_t *)0x64000010))
//#define SFMCMD  (*((volatile uint32_t *)0x64000014))
#define SFMCST  (*((volatile uint32_t *)0x64000018))
#define SFMSIC  (*((volatile uint32_t *)0x64000020))
#define SFMSAC  (*((volatile uint32_t *)0x64000024))
#define SFMSDC  (*((volatile uint32_t *)0x64000028))
#define SFMSPC  (*((volatile uint32_t *)0x64000030))
#define SFMPMD  (*((volatile uint32_t *)0x64000034))
#define SFMDMA  (*((volatile uint32_t *)0x6400080C))
#define SFMCRPT (*((volatile uint32_t *)0x64000810))
	

//#define BYTE_FIELD(value, byte) ((adr & (0xFF << 8 * byte )) >> 8 * byte)
#define BYTE_FIELD(value, byte) ((value & (0xFF << 8 * byte )) >> 8 * byte)

/**************************************************************************
* Bit Fields defination
**************************************************************************/
// Status Register-1
#define BUSY_Pos  (0x00U)
#define BUSY_Msk  (0x01U)
#define WEL_Pos   (0x01U)
#define WEL_Msk   (0x02U)
#define BP_Pos    (0x04U)
#define BP_Msk    (0x30U)
#define TB_Pos    (0x06U)
#define TB_Msk    (0x40U)
#define BPL_Pos   (0x07U)
#define BPS_Msk   (0x80U)
// Status Register-2
#define QE_Pos    (0x01U)
#define QE_Msk    (0x02U)

/**************************************************************************
* Typedef defination
**************************************************************************/
typedef enum
{
	STATUS_REG1 = 1,
	STATUS_REG2 = 2
} status_t;

/**************************************************************************
* Global functions
**************************************************************************/
void Qspi_DirectMod(void);
void Qspi_ROMMod(void);
void Qspi_Data_Input(uint8_t QspiData);
void Qspi_BusOff(void);
uint8_t Qspi_ReadData(void);
uint8_t Qspi_StatusFlag(uint8_t QFlag);


void SoftReset(void);

uint8_t ReadStatus(status_t name);

void WriteStatus(status_t name, uint8_t data);

void EnableQPI(void);

void DisableQPI(void);
void EraseSector(uint32_t adr);

void EraseBlock(uint32_t adr);

void EraseChip(void);

void ProgramPage(uint32_t adr, uint32_t sz, uint8_t *buf);

#endif
