#include "cmp.h"
#include "cgc.h"
#define TESOPT  *((volatile uint8_t *)(0x4004384C))  /*!< (@ 0x4004384C) PGA protect register                          */


/***********************************************************************************************************************
* Function Name: CMP_Input_Select
* @brief  This function sets the positive and negative side input of Comparator.
* @param  ch - specify the comp channel number
* @param  positive_side - specify the positive side input of Comparator
* @param  negative_side - specify the negative side input of Comparator
* @return None
***********************************************************************************************************************/
void CMP_Input_Select(CMP_Channel_t ch, CMP_Side_t positive_side, CMP_Side_t negative_side)
{
	if(ch & 0x01)
	{
			CMP0->COMP0SEL = (positive_side << CMP0_COMP0SEL_CMPSEL_Pos) | (negative_side << CMP0_COMP0SEL_C0REFS_Pos);

	}
	if(ch & 0x02)
	{
			CMP1->COMP1SEL = (positive_side << CMP1_COMP1SEL_CMPSEL_Pos) | (negative_side << CMP1_COMP1SEL_C1REFS_Pos);
	}
	if(ch & 0x03)
	{
			CMP2->COMP2SEL = (positive_side << CMP2_COMP2SEL_CMPSEL_Pos) | (negative_side << CMP2_COMP2SEL_C2REFS_Pos);
	}
	if(ch & 0x04)
	{
			CMP3->COMP3SEL = (positive_side << CMP3_COMP3SEL_CMPSEL_Pos) | (negative_side << CMP3_COMP3SEL_C3REFS_Pos);
	}
}

/***********************************************************************************************************************
* Function Name: CMP_Init
* @brief  This function initilize the CMP.
***********************************************************************************************************************/
void CMP_Init(CMP_InitTypeDef* CMP_InitStruct)
{
	CGC_PERAPeriphClockCmd(CGC_PERAPeriph_CMPPGA,ENABLE);

	if(CMP_InitStruct->CMP_Channel & 0x01)
	{

		CMP_Input_Select  (CMP_InitStruct->CMP_Channel, CMP_InitStruct->CMP_PositiveSide, CMP_InitStruct->CMP_NegativeSide);
		CMP0->COMP0OCR = CMP_InitStruct->CMP_Mod << 7 | CMP_InitStruct->CMP_Polarity<<2 | CMP_InitStruct->CMP_Out2port<<1 | CMP_InitStruct->CMP_Int;        
				
		if(CMP_InitStruct->CMP_Int)
		{
			CMP0->COMP0FIR &= ~(CMP0_COMP0FIR_C0EDG_Msk | CMP0_COMP0FIR_C0EPO_Msk);	
			if(CMP_InitStruct->CMP_IntReq == CMP_IntReq_Both)
			{
				CMP0->COMP0FIR |= 0x08;
			}
			else
			{
				CMP0->COMP0FIR |= (CMP_InitStruct->CMP_IntReq << 2);
			}
		}		
		CMP0->COMP0FIR |= CMP_InitStruct->CMP_Filter;
		CMP0->COMP0MDR = _01_COMPx_OPERATION_ENABLED;
		CMP0->COMP0HY |= ((CMP_InitStruct->CMP_HyVal << 4) | CMP_InitStruct->CMP_HyMode);
	}
	if(CMP_InitStruct->CMP_Channel & 0x02)
	{

		CMP_Input_Select  (CMP_InitStruct->CMP_Channel, CMP_InitStruct->CMP_PositiveSide, CMP_InitStruct->CMP_NegativeSide);
		CMP1->COMP1OCR = CMP_InitStruct->CMP_Mod << 7 | CMP_InitStruct->CMP_Polarity<<2 | CMP_InitStruct->CMP_Out2port<<1 | CMP_InitStruct->CMP_Int;        
				
		if(CMP_InitStruct->CMP_Int)
		{
			CMP1->COMP1FIR &= ~(CMP1_COMP1FIR_C1EDG_Msk | CMP1_COMP1FIR_C1EPO_Msk);	
			if(CMP_InitStruct->CMP_IntReq == CMP_IntReq_Both)
			{
				CMP1->COMP1FIR |= 0x08;
			}
			else
			{
				CMP1->COMP1FIR |= (CMP_InitStruct->CMP_IntReq << 2);
			}
		}		
		CMP1->COMP1FIR |= CMP_InitStruct->CMP_Filter;
		CMP1->COMP1MDR = _01_COMPx_OPERATION_ENABLED;
		CMP1->COMP1HY |= ((CMP_InitStruct->CMP_HyVal << 4) | CMP_InitStruct->CMP_HyMode);
	}
		if(CMP_InitStruct->CMP_Channel & 0x03)
	{

		CMP_Input_Select  (CMP_InitStruct->CMP_Channel, CMP_InitStruct->CMP_PositiveSide, CMP_InitStruct->CMP_NegativeSide);
		CMP2->COMP2OCR = CMP_InitStruct->CMP_Mod << 7 | CMP_InitStruct->CMP_Polarity<<2 | CMP_InitStruct->CMP_Out2port<<1 | CMP_InitStruct->CMP_Int;        
				
		if(CMP_InitStruct->CMP_Int)
		{
			CMP2->COMP2FIR &= ~(CMP2_COMP2FIR_C2EDG_Msk | CMP2_COMP2FIR_C2EPO_Msk);	
			if(CMP_InitStruct->CMP_IntReq == CMP_IntReq_Both)
			{
				CMP2->COMP2FIR |= 0x08;
			}
			else
			{
				CMP2->COMP2FIR |= (CMP_InitStruct->CMP_IntReq << 2);
			}
		}		
		CMP2->COMP2FIR |= CMP_InitStruct->CMP_Filter;
		CMP2->COMP2MDR = _01_COMPx_OPERATION_ENABLED;
		CMP2->COMP2HY |= ((CMP_InitStruct->CMP_HyVal << 4) | CMP_InitStruct->CMP_HyMode);
	}
		if(CMP_InitStruct->CMP_Channel & 0x04)
	{

		CMP_Input_Select  (CMP_InitStruct->CMP_Channel, CMP_InitStruct->CMP_PositiveSide, CMP_InitStruct->CMP_NegativeSide);
		CMP3->COMP3OCR = CMP_InitStruct->CMP_Mod << 7 | CMP_InitStruct->CMP_Polarity<<2 | CMP_InitStruct->CMP_Out2port<<1 | CMP_InitStruct->CMP_Int;        
				
		if(CMP_InitStruct->CMP_Int)
		{
			CMP3->COMP3FIR &= ~(CMP3_COMP3FIR_C3EDG_Msk | CMP3_COMP3FIR_C3EPO_Msk);	
			if(CMP_InitStruct->CMP_IntReq == CMP_IntReq_Both)
			{
				CMP3->COMP3FIR |= 0x08;
			}
			else
			{
				CMP3->COMP3FIR |= (CMP_InitStruct->CMP_IntReq << 2);
			}
		}		
		CMP3->COMP3FIR |= CMP_InitStruct->CMP_Filter;
		CMP3->COMP3MDR = _01_COMPx_OPERATION_ENABLED;
		CMP3->COMP3HY |= ((CMP_InitStruct->CMP_HyVal << 4) | CMP_InitStruct->CMP_HyMode);
	}

}

/***********************************************************************************************************************
* Function Name: CMP_Start
* @brief  This function starts the Comparator.
* @param  ch - specify the channel
***********************************************************************************************************************/
void CMP_Start(CMP_Channel_t ch)
{
    if(ch & 0x01)
    {
        NVIC_ClearPendingIRQ(CMP0_IRQn);    // clear INTCMP0 interrupt flag 
        NVIC_EnableIRQ(CMP0_IRQn);          // enable INTCMP0 interrupt 
        CMP0->COMP0MDR = _01_COMPx_OPERATION_ENABLED;
	}
    if(ch & 0x02)
    {
        NVIC_ClearPendingIRQ(CMP1_IRQn);    // clear INTCMP1 interrupt flag 
        NVIC_EnableIRQ(CMP1_IRQn);          // enable INTCMP1 interrupt 
        CMP1->COMP1MDR = _01_COMPx_OPERATION_ENABLED;
	}
		    if(ch & 0x03)
    {
        NVIC_ClearPendingIRQ(CMP2_IRQn);    // clear INTCMP2 interrupt flag 
        NVIC_EnableIRQ(CMP2_IRQn);          // enable INTCMP2 interrupt 
        CMP2->COMP2MDR = _01_COMPx_OPERATION_ENABLED;
	}
		    if(ch & 0x04)
    {
        NVIC_ClearPendingIRQ(CMP3_IRQn);    // clear INTCMP3 interrupt flag 
        NVIC_EnableIRQ(CMP3_IRQn);          // enable INTCMP3 interrupt 
        CMP3->COMP3MDR = _01_COMPx_OPERATION_ENABLED;
	}
		
}
/***********************************************************************************************************************
* Function Name: CMP_Stop
* @brief  This function starts the Comparator.
* @param  ch - specify the channel
***********************************************************************************************************************/
void CMP_Stop(CMP_Channel_t ch)
{
    if(ch & 0x01)
    {
        CMP0->COMP0MDR &= ~_01_COMPx_OPERATION_ENABLED;
        NVIC_DisableIRQ(CMP0_IRQn);         // disable INTCMP0 interrupt 
        NVIC_ClearPendingIRQ(CMP0_IRQn);    // clear INTCMP0 interrupt flag 
	}
    if(ch & 0x02)
    {
        CMP1->COMP1MDR &= ~_01_COMPx_OPERATION_ENABLED;
        NVIC_DisableIRQ(CMP1_IRQn);         // disable INTCMP1 interrupt
        NVIC_ClearPendingIRQ(CMP1_IRQn);    // clear INTCMP1 interrupt flag
	}
		    if(ch & 0x03)
    {
        CMP2->COMP2MDR &= ~_01_COMPx_OPERATION_ENABLED;
        NVIC_DisableIRQ(CMP2_IRQn);         // disable INTCMP1 interrupt
        NVIC_ClearPendingIRQ(CMP2_IRQn);    // clear INTCMP1 interrupt flag
	}
		    if(ch & 0x04)
    {
        CMP3->COMP3MDR &= ~_01_COMPx_OPERATION_ENABLED;
        NVIC_DisableIRQ(CMP3_IRQn);         // disable INTCMP1 interrupt
        NVIC_ClearPendingIRQ(CMP3_IRQn);    // clear INTCMP1 interrupt flag
	}
}

