#include "dac.h"
#include "cgc.h"


/**
  * @brief  Start the specified DAC channel .
  * @param  DACx:DAC0, DAC1
  * @retval None
  */
void DAC_Start(DAC_TypeDef DACx)
{
	if(DACx == DAC0)
	{
		DAC->DAM |= DAC_DAM_DACE0_Msk;
	}
	else
	{
		DAC->DAM |= DAC_DAM_DACE1_Msk;
	}
}

/**
  * @brief  Stop the specified DAC channel .
* @param  DACx:DAC0, DAC1
  * @retval None
  */
void DAC_Stop(DAC_TypeDef DACx)
{
	if(DACx == DAC0)
	{
		DAC->DAM &= ~DAC_DAM_DACE0_Msk;
	}
	else
	{
		DAC->DAM &= ~DAC_DAM_DACE1_Msk;
	}
}

/**
  * @brief  Select DAC output to AOx port or ACMP .
  * @param  DACx:DAC0, DAC1
  * @param  OutMod:
  *              DAC_OutToPort: DAC out to AOx
  *              DAC_OutToAcmp: DAC out to ACMP Negative
  * @retval None
  */
void DAC_SelectOut(DAC_TypeDef DACx,uint8_t OutMod)
{
	if(DACx == DAC0)
	{
		DAC->DAM |= (OutMod << DAC_DAM_DACOENB0_Pos);
	}
	else
	{
		DAC->DAM |= (OutMod << DAC_DAM_DACOENB1_Pos);
	}
}

/**
  * @brief  Select DAC output mode .
  * @param  DACx:DAC0, DAC1
  * @param  Mod:
  *              DAC_Normal: DAC normal run
  *              DAC_RealOut: DAC real output
  * @retval None
  */
void DAC_Mode(DAC_TypeDef DACx,uint8_t Mod)
{
	if(DACx == DAC0)
	{
		DAC->DAM |= (Mod << DAC_DAM_DAMD0_Pos);
	}
	else
	{
		DAC->DAM |= (Mod << DAC_DAM_DAMD1_Pos);
	}
}

