#include "pga.h"
#include "cgc.h"

#define TESOPT  *((volatile uint8_t *)(0x4004384C))  /*!< (@ 0x4004384C) PGA protect register                          */
#define TESCTL 	*((volatile uint8_t *)(0x4004384D))  /*!< (@ 0x4004384D) PGA output control register                   */

/***********************************************************************************************************************
* Function Name: PGA_Init
* @brief  This function init the programmable gain amplifier.
* @param  PGA_InitTypeDef
* @return  none 
***********************************************************************************************************************/
void PGA_Init(PGA_InitTypeDef *PGA_InitStruct)
{
	
	if(PGA_InitStruct->PGA_Channel == PGA00)
	{
		CGC_PERAPeriphClockCmd(CGC_PERAPeriph_ADC0, ENABLE);
	}
	else
	{
		CGC_PERAPeriphClockCmd(CGC_PERAPeriph_ADC1, ENABLE);
	}

	if(PGA_InitStruct->PGA_Channel == PGA00)
	{
		ADC0->ADPGACR = (PGA_InitStruct->PGA_Mod << ADC0_ADPGACR_P00MODE_Pos) | PGA_InitStruct->PGA_Out;
		ADC0->ADPGAGS = PGA_InitStruct->PGA_Gain;
		ADC0->ADPGADCR = PGA_InitStruct->PGA_Gnd << ADC0_ADPGADCR_P00GNDSEL_Pos;
	}
	else if(PGA_InitStruct->PGA_Channel == PGA10)
	{
		ADC1->ADPGACR = (PGA_InitStruct->PGA_Mod << ADC1_ADPGACR_P10MODE_Pos) | PGA_InitStruct->PGA_Out;
		ADC1->ADPGAGS = PGA_InitStruct->PGA_Gain;
		ADC1->ADPGADCR = PGA_InitStruct->PGA_Gnd << ADC1_ADPGADCR_P10GNDSEL_Pos;
	}
	else if(PGA_InitStruct->PGA_Channel == PGA11)
	{
		ADC1->ADPGACR = (PGA_InitStruct->PGA_Mod << ADC1_ADPGACR_P11MODE_Pos) | (PGA_InitStruct->PGA_Out << ADC1_ADPGACR_P11SW_Pos);
		ADC1->ADPGAGS = PGA_InitStruct->PGA_Gain << ADC1_ADPGAGS_P11GAIN_Pos;
		ADC1->ADPGADCR = PGA_InitStruct->PGA_Gnd << ADC1_ADPGADCR_P11GNDSEL_Pos;
	}
	else if(PGA_InitStruct->PGA_Channel == PGA12)
	{
		ADC1->ADPGACR = (PGA_InitStruct->PGA_Mod << ADC1_ADPGACR_P12MODE_Pos) | (PGA_InitStruct->PGA_Out << ADC1_ADPGACR_P12SW_Pos);
		ADC1->ADPGAGS = PGA_InitStruct->PGA_Gain << ADC1_ADPGAGS_P12GAIN_Pos;
		ADC1->ADPGADCR = PGA_InitStruct->PGA_Gnd << ADC1_ADPGADCR_P12GNDSEL_Pos;
	}
		
	PGA_Start(PGA_InitStruct->PGA_Channel);  //start PGA
}

/***********************************************************************************************************************
* Function Name: PGA_Start
* @brief  This function starts the  programmable gain amplifier.
* @param  ch - specify the pga channel number
* @return  none 
***********************************************************************************************************************/
void PGA_Start(PGA_Channel_t ch)
{
	if(ch == 0x00)     
	{        
		ADC0->ADPGACR |= ADC0_ADPGACR_P00EN_Msk;     // PGA0 operation enabled 
	}
	else
	{
		ADC1->ADPGACR |= 1 << (4*ch-2);     // PGA1 operation enabled 
	}
}

/***********************************************************************************************************************
* Function Name: PGA_Stop
* @brief  This function stops the  programmable gain amplifier.
* @param  ch - specify the pga channel number
* @return  none 
***********************************************************************************************************************/
void PGA_Stop(PGA_Channel_t ch)
{
	
	if(ch == 0x00)     
	{        
		ADC0->ADPGACR &= ~ADC0_ADPGACR_P00EN_Msk;     // PGA0 operation disabled 
	}
	else
	{
		ADC1->ADPGACR &= ~(1 << (4*ch-2));     // PGA1 operation disabled 
	}
}

