#include "pwm.h"
#include "cgc.h"

/**
  * @brief  Enable to write WP bit .
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17    
  * @retval None
  */
void PWM_Lock(GPT_Type * GPTx)
{
//	GPTx->GTWP |= PWM_WPKey;								//д WP λ
	GPTx->GTWP = GPT0_GTWP00_WP_Msk | PWM_WPKey; 			//ֹдĴ
	
//	GPTx->GTWP &= ~GPT0_GTWP00_PRKEY_Msk;  	//ֹдWPλ
}

/**
  * @brief  Disable to write WP bit .
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17    
  * @retval None
  */
void PWM_UnLock(GPT_Type * GPTx)
{
	GPTx->GTWP = PWM_WPKey;								//д WP λ
//	GPTx->GTWP &= ~GPT0_GTWP00_WP_Msk; 			//дĴ
	
//	GPTx->GTWP &= ~GPT0_GTWP00_PRKEY_Msk;  	//ֹдWPλ
}

/**
  * @brief  Start PWM counter by soft register.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17    
  * @retval None
  */
void PWM_Counter_SoftStart(GPT_Type * GPTx)
{
	PWM_UnLock(GPTx);								//дĴ
	
	if(GPTx == GPT00)
		GPTx->GTSTR |= (1<<GPT0_GTSTR00_CSTRT_Pos); 			
	else if(GPTx == GPT01)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+1)); 			
	else if(GPTx == GPT02)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+2)); 			
	else if(GPTx == GPT03)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+3)); 			
	else if(GPTx == GPT10)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+4)); 			
	else if(GPTx == GPT11)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+5)); 			
	else if(GPTx == GPT12)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+6)); 			
	else if(GPTx == GPT13)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+7)); 			
	else if(GPTx == GPT14)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+8)); 			
	else if(GPTx == GPT15)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+9)); 			
	else if(GPTx == GPT16)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+10)); 			
	else if(GPTx == GPT17)
		GPTx->GTSTR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+11)); 			
	PWM_Lock(GPTx);								//ֹдĴ
}

/**
  * @brief  Stop PWM counter  by soft register.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17    
  * @retval None
  */
void PWM_Counter_SoftStop(GPT_Type * GPTx)
{
	PWM_UnLock(GPTx);								//дĴ
	
	if(GPTx == GPT00)
		GPTx->GTSTP |= (1<<GPT0_GTSTR00_CSTRT_Pos); 			
	else if(GPTx == GPT01)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+1)); 			
	else if(GPTx == GPT02)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+2)); 			
	else if(GPTx == GPT03)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+3)); 			
	else if(GPTx == GPT10)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+4)); 			
	else if(GPTx == GPT11)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+5)); 			
	else if(GPTx == GPT12)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+6)); 			
	else if(GPTx == GPT13)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+7)); 			
	else if(GPTx == GPT14)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+8)); 			
	else if(GPTx == GPT15)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+9)); 			
	else if(GPTx == GPT16)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+10)); 			
	else if(GPTx == GPT17)
		GPTx->GTSTP |= (1<<(GPT0_GTSTR00_CSTRT_Pos+11)); 			
	PWM_Lock(GPTx);								//ֹдĴ
}

/**
  * @brief  Clear PWM counter by soft register.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17    
  * @retval None
  */
void PWM_Counter_SoftClear(GPT_Type * GPTx)
{
	PWM_UnLock(GPTx);								//дĴ
	
	if(GPTx == GPT00)
		GPTx->GTCLR |= (1<<GPT0_GTSTR00_CSTRT_Pos); 			
	else if(GPTx == GPT01)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+1)); 	
	else if(GPTx == GPT02)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+2)); 	
	else if(GPTx == GPT03)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+3)); 	
	else if(GPTx == GPT10)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+4)); 	
	else if(GPTx == GPT11)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+5)); 	
	else if(GPTx == GPT12)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+6)); 	
	else if(GPTx == GPT13)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+7)); 	
	else if(GPTx == GPT14)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+8)); 	
	else if(GPTx == GPT15)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+9)); 	
	else if(GPTx == GPT16)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+10)); 	
	else if(GPTx == GPT17)
		GPTx->GTCLR |= (1<<(GPT0_GTSTR00_CSTRT_Pos+11)); 			
	PWM_Lock(GPTx);								//ֹдĴ
}

/**
  * @brief  Select PWM counter triger start source.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Src: PWMSrc_TypeDef
  *             PWMSrc_GTETRGA_R........PWMSrc_Soft_GTSTR
  * @param  Stat: 
  *             ENABLE,DISABLE
  * @retval None
  */
void PWM_Start_Source(GPT_Type * GPTx,PWMSrc_TypeDef Src,uint8_t Stat)
{
	PWM_UnLock(GPTx);								//дĴ
	if(Stat == ENABLE)
		GPTx->GTSSR |= 1<<Src ;							//Դ
	else
		GPTx->GTSSR &= ~(1<<Src);							//ֹԴ
	
	PWM_Lock(GPTx);								//ֹдĴ	
}

/**
  * @brief  Select PWM counter triger stop source.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Src: PWMSrc_TypeDef
  *             PWMSrc_GTETRGA_R........PWMSrc_Soft_GTSTR
  * @param  Stat: 
  *             ENABLE,DISABLE
  * @retval None
  */
void PWM_Stop_Source(GPT_Type * GPTx,PWMSrc_TypeDef Src,uint8_t Stat)
{
	PWM_UnLock(GPTx);								//дĴ
	if(Stat == ENABLE)
		GPTx->GTPSR |= 1<<Src ;							//Դ
	else
		GPTx->GTPSR &= ~(1<<Src);							//ֹԴ
	
	PWM_Lock(GPTx);								//ֹдĴ	
}

/**
  * @brief  Select PWM counter value clear source.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Src: PWMSrc_TypeDef
  *             PWMSrc_GTETRGA_R........PWMSrc_Soft_GTSTR
  * @param  Stat: 
  *             ENABLE,DISABLE
  * @retval None
  */
void PWM_Clear_Source(GPT_Type * GPTx,PWMSrc_TypeDef Src,uint8_t Stat)
{
	PWM_UnLock(GPTx);								//дĴ
	if(Stat == ENABLE)
		GPTx->GTCSR |= 1<<Src ;							//Դ
	else
		GPTx->GTCSR &= ~(1<<Src);							//ֹԴ
	
	PWM_Lock(GPTx);								//ֹдĴ	
}

/**
  * @brief  Select PWM counter UP count source.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Src: PWMSrc_TypeDef
  *             PWMSrc_GTETRGA_R........PWMSrc_ELCD
  *             (without PWMSrc_Soft_GTSTR)
  * @param  Stat: 
  *             ENABLE,DISABLE
  * @retval None
  */
void PWM_UpCount_Source(GPT_Type * GPTx,PWMSrc_TypeDef Src,uint8_t Stat)
{
	PWM_UnLock(GPTx);								//дĴ
	if(Stat == ENABLE)
		GPTx->GTUPSR |= 1<<Src ;							//Դ
	else
		GPTx->GTUPSR &= ~(1<<Src);							//ֹԴ
	
	PWM_Lock(GPTx);								//ֹдĴ	
}

/**
  * @brief  Select PWM counter Down count source.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Src: PWMSrc_TypeDef
  *             PWMSrc_GTETRGA_R........PWMSrc_ELCD
  *             (without PWMSrc_Soft_GTSTR)
  * @param  Stat: 
  *             ENABLE,DISABLE
  * @retval None
  */
void PWM_DownCount_Source(GPT_Type * GPTx,PWMSrc_TypeDef Src,uint8_t Stat)
{
	PWM_UnLock(GPTx);								//дĴ
	if(Stat == ENABLE)
		GPTx->GTDNSR |= 1<<Src ;							//Դ
	else
		GPTx->GTDNSR &= ~(1<<Src);							//ֹԴ
	
	PWM_Lock(GPTx);								//ֹдĴ	
}

/**
  * @brief  Select PWM capture A start source.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Src: PWMSrc_TypeDef
  *             PWMSrc_GTETRGA_R........PWMSrc_ELCD
  *             (without PWMSrc_Soft_GTSTR)
  * @param  Stat: 
  *             ENABLE,DISABLE
  * @retval None
  */
void PWM_CaptureA_Source(GPT_Type * GPTx,PWMSrc_TypeDef Src,uint8_t Stat)
{
	PWM_UnLock(GPTx);								//дĴ
	if(Stat == ENABLE)
		GPTx->GTICASR |= 1<<Src ;							//Դ
	else
		GPTx->GTICASR &= ~(1<<Src);							//ֹԴ
	
	PWM_Lock(GPTx);								//ֹдĴ	
}

/**
  * @brief  Select PWM capture B start source.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Src: PWMSrc_TypeDef
  *             PWMSrc_GTETRGA_R........PWMSrc_ELCD
  *             (without PWMSrc_Soft_GTSTR)
  * @param  Stat: 
  *             ENABLE,DISABLE
  * @retval None
  */
void PWM_CaptureB_Source(GPT_Type * GPTx,PWMSrc_TypeDef Src,uint8_t Stat)
{
	PWM_UnLock(GPTx);								//дĴ
	if(Stat == ENABLE)
		GPTx->GTICBSR |= 1<<Src ;							//Դ
	else
		GPTx->GTICBSR &= ~(1<<Src);							//ֹԴ
	
	PWM_Lock(GPTx);								//ֹдĴ	
}


/**
  * @brief  Start count.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17    
  * @retval None
  */
void PWM_Start(GPT_Type * GPTx)
{
	PWM_UnLock(GPTx);								//дĴ

	GPTx->GTCR |= GPT0_GTCR00_CST_Msk; 			
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  Stop count.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17    
  * @retval None
  */
void PWM_Stop(GPT_Type * GPTx)
{
	PWM_UnLock(GPTx);								//дĴ

	GPTx->GTCR &= ~GPT0_GTCR00_CST_Msk; 			
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM wave mode select.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  WavMod: 
  *           PWM_SawtoothWave     : Ⲩ PWM ģʽ
  *           PWM_SToothOneTimeWave  : Ⲩһģʽ
  *           PWM_TriWave_Mode1		:  ȴ 32 λ
  *           PWM_TriWave_Mode2		:  崦 32 λ
  *           PWM_TriWave_Mode3		:  ȴ 64 λ
  * @retval None
  */
void PWM_WaveMode(GPT_Type * GPTx,GTIOMod_FunState WavMod)
{
	uint8_t Temp=0;
	
	PWM_UnLock(GPTx);								//дĴ
  Temp = GPTx->GTCR;
	Temp &= ~GPT0_GTCR00_MD_Msk; 
	Temp |= WavMod << GPT0_GTCR00_MD_Pos;
	GPTx->GTCR = Temp; 		
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM timer div frequency.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Freq: 
  *            PWM_Div_1    :1 Ƶ
  *            PWM_Div_4    :4 Ƶ
  *            PWM_Div_16   :16 Ƶ
  *            PWM_Div_64   :64 Ƶ
  *            PWM_Div_256  :256 Ƶ
  *            PWM_Div_1024 :1024 Ƶ
  * @retval None
  */
void PWM_TimerFrequency(GPT_Type * GPTx,GTIODiv_FunState Freq)
{
	uint8_t Temp=0;
	
	PWM_UnLock(GPTx);								//дĴ
  Temp = GPTx->GTCR;
	Temp &= ~GPT0_GTCR00_TPCS_Msk; 
	Temp |= Freq << GPT0_GTCR00_TPCS_Pos;
	GPTx->GTCR = Temp; 		
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM timer count direction.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  CountDir


: 
  *            PWM_TimerDec    :ݼ
  *            PWM_TimerInc    :
  * @retval None
  */
void PWM_TimerDir(GPT_Type * GPTx,GTIODir_FunState CountDir)
{
	uint8_t Temp=0;
	
	PWM_UnLock(GPTx);								//дĴ
  Temp = GPTx->GTUDDTYC;
	Temp &= ~GPT0_GTUDDTYC00_UD_Msk; 
	Temp |= CountDir << GPT0_GTUDDTYC00_UD_Pos;
	GPTx->GTUDDTYC = Temp; 		
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM timer count direction control.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Cstatus: 
  *            ENABLE,DISABLE
  * @retval None
  */
void PWM_TimerDir_Control(GPT_Type * GPTx,uint8_t Cstatus)
{
	
	PWM_UnLock(GPTx);								//дĴ
 	if(Cstatus == ENABLE)
		GPTx->GTUDDTYC |= GPT0_GTUDDTYC00_UDF_Msk;   //ǿƼ
	else
		GPTx->GTUDDTYC &= ~GPT0_GTUDDTYC00_UDF_Msk;   //ǿƼ
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCA out duty set.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  VDuty: 
  *            PWM_GTIOCx_Duty0: 0% ռձ
  *            PWM_GTIOCx_Duty100100% ռձ
  * @retval None
  */
void PWM_GTIOCA_Duty(GPT_Type * GPTx,GTIOForceDuty_FunState VDuty)
{
	uint8_t Temp=0;
	
	PWM_UnLock(GPTx);								//дĴ
	Temp = GPTx->GTUDDTYC;
	Temp &= ~GPT0_GTUDDTYC00_OADTY_Msk;
	Temp |= VDuty<<GPT0_GTUDDTYC00_OADTY_Pos;
	
	GPTx->GTUDDTYC = Temp;   
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCA out duty set control.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Cstatus: 
  *            ENABLEǿGTIOCAռձ
  *            DISABLEǿGTIOCAռձ
  * @retval None
  */
void PWM_GTIOCA_DutyCtrl(GPT_Type * GPTx,uint8_t Cstatus)
{
	
	PWM_UnLock(GPTx);								//дĴ
 	if(Cstatus == ENABLE)
		GPTx->GTUDDTYC |= GPT0_GTUDDTYC00_OADTYF_Msk;   //ǿGTIOCAռձ
	else
		GPTx->GTUDDTYC &= ~GPT0_GTUDDTYC00_OADTYF_Msk;   //ǿGTIOCAռձ
	
	PWM_Lock(GPTx);								//ֹдĴ		
}


/**
  * @brief  PWM GTIOCA out after cancel duty
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Freq: 
  *            ENABLEʹGTIOA[3:2]ΪȡռձȺGTIOCA	
  *            DISABLEʹGTIOA[3:2]ΪȡռձȺεıȽƥֵ
  * @retval None
  */
void PWM_GTIOCA_Output(GPT_Type * GPTx,uint8_t OutSta)
{
	
	PWM_UnLock(GPTx);								//дĴ
 	if(OutSta == ENABLE)
		GPTx->GTUDDTYC &= ~GPT0_GTUDDTYC00_OADTYR_Msk;   //ʹGTIOA[3:2]ΪȡռձȺGTIOCA		
	else
		GPTx->GTUDDTYC |= GPT0_GTUDDTYC00_OADTYR_Msk;   //ʹGTIOA[3:2]ΪȡռձȺεıȽƥֵ
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCB out duty set.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  VDuty: 
  *            PWM_GTIOCx_Duty0: 0% ռձ
  *            PWM_GTIOCx_Duty100100% ռձ
  * @retval None
  */
void PWM_GTIOCB_Duty(GPT_Type * GPTx,uint8_t VDuty)
{
	uint8_t Temp=0;
	
	PWM_UnLock(GPTx);								//дĴ
	Temp = GPTx->GTUDDTYC;
	Temp &= ~GPT0_GTUDDTYC00_OBDTY_Msk;
	Temp |= VDuty<<GPT0_GTUDDTYC00_OBDTY_Pos;
	
	GPTx->GTUDDTYC = Temp;   
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCB out duty set control.
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Cstatus: 
  *            ENABLEǿGTIOCBռձ
  *            DISABLEǿGTIOCBռձ
  * @retval None
  */
void PWM_GTIOCB_DutyCtrl(GPT_Type * GPTx,uint8_t Cstatus)
{
	
	PWM_UnLock(GPTx);								//дĴ
 	if(Cstatus == ENABLE)
		GPTx->GTUDDTYC |= GPT0_GTUDDTYC00_OBDTYF_Msk;   //ǿGTIOCBռձ
	else
		GPTx->GTUDDTYC &= ~GPT0_GTUDDTYC00_OBDTYF_Msk;   //ǿGTIOCBռձ
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCB out after cancel duty
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Freq: 
  *            ENABLEʹGTIOB[3:2]ΪȡռձȺGTIOCB	
  *            DISABLEʹGTIOB[3:2]ΪȡռձȺεıȽƥֵ
  * @retval None
  */
void PWM_GTIOCB_Output(GPT_Type * GPTx,uint8_t OutSta)
{
	
	PWM_UnLock(GPTx);								//дĴ
 	if(OutSta == ENABLE)
		GPTx->GTUDDTYC &= ~GPT0_GTUDDTYC00_OBDTYR_Msk;   //ʹGTIOB[3:2]ΪȡռձȺGTIOCB		
	else
		GPTx->GTUDDTYC |= GPT0_GTUDDTYC00_OBDTYR_Msk;   //ʹGTIOB[3:2]ΪȡռձȺεıȽƥֵ
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCA/GTIOCB IO function select
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Chann: 
  *            PWM_GTIOCA, PWM_GTIOCB
  * @param  Freq: 
   *            PWM_GTIOCx_LowCEndKpCompareMatch
									.......
  *            PWM_GTIOCx_HighSwOutSwCompareMatch            
  * @retval None
  */
void PWM_GTIOCx_Func(GPT_Type * GPTx,uint8_t Chann,GTIOFunc_FunState Func)
{
	
	PWM_UnLock(GPTx);								//дĴ

	if(Chann == PWM_GTIOCA)
	{
		GPTx->GTIOR &= ~GPT0_GTIOR00_GTIOA_Msk;   
		GPTx->GTIOR |= Func;
	}
  else
	{
		GPTx->GTIOR &= ~GPT0_GTIOR00_GTIOB_Msk;   
		GPTx->GTIOR |= Func << GPT0_GTIOR00_GTIOB_Pos;
	}
	
	PWM_Lock(GPTx);								//ֹдĴ		
}


/**
  * @brief  PWM GTIOCA/GTIOCB IO status when stop count
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Chann: 
  *            PWM_GTIOCA, PWM_GTIOCB
  * @param  IOStat: 
  *            IOLowGTIOCx
  *            IOHighGTIOCx       
  * @retval None
  */
void PWM_GTIOCxIO_StopCountStat(GPT_Type * GPTx,uint8_t Chann,IO_FunState IOStat)
{
	
	PWM_UnLock(GPTx);								//дĴ

	if(Chann == PWM_GTIOCA)
	{
		if(IOStat == IOLow)
		  GPTx->GTIOR &= ~GPT0_GTIOR00_OADFLT_Msk;   
		else
			GPTx->GTIOR |= GPT0_GTIOR00_OADFLT_Msk;   
	}
  else
	{
		if(IOStat == IOLow)
		  GPTx->GTIOR &= ~GPT0_GTIOR00_OBDFLT_Msk;   
		else
			GPTx->GTIOR |= GPT0_GTIOR00_OBDFLT_Msk; 
	}
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCA/GTIOCB IO status when stop count
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Chann: 
  *            PWM_GTIOCA, PWM_GTIOCB
  * @param  IOStat: 
  *            IOLowGTIOCx IOʼֹͣʱƽΪĴֵ
  *            IOHighGTIOCx IOʼֹͣʱGTIOCxն˵״̬     
  * @retval None
  */
void PWM_GTIOCxIO_HoldStat(GPT_Type * GPTx,uint8_t Chann,IO_FunState IOStat)
{
	
	PWM_UnLock(GPTx);								//дĴ

	if(Chann == PWM_GTIOCA)
	{
		if(IOStat == IOLow)
		  GPTx->GTIOR &= ~GPT0_GTIOR00_OAHLD_Msk;   
		else
			GPTx->GTIOR |= GPT0_GTIOR00_OAHLD_Msk;   
	}
  else
	{
		if(IOStat == IOLow)
		  GPTx->GTIOR &= ~GPT0_GTIOR00_OBHLD_Msk;   
		else
			GPTx->GTIOR |= GPT0_GTIOR00_OBHLD_Msk; 
	}
	
	PWM_Lock(GPTx);								//ֹдĴ		
}


/**
  * @brief  PWM GTIOCA/GTIOCB IO output enable/disable
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Chann: 
  *            PWM_GTIOCA, PWM_GTIOCB
  * @param  IOStat: 
  *            ENABLEGTIOCx IOʹ
  *            DISABLEGTIOCx IOֹ     
  * @retval None
  */
void PWM_GTIOCxIO_OutCtrl(GPT_Type * GPTx,uint8_t Chann,FunctionalState IOStat)
{
	
	PWM_UnLock(GPTx);								//дĴ

	if(Chann == PWM_GTIOCA)
	{
		if(IOStat == ENABLE)
			GPTx->GTIOR |= GPT0_GTIOR00_OAE_Msk;	     
		else
			GPTx->GTIOR &= ~GPT0_GTIOR00_OAE_Msk;   
	}
  else
	{
		if(IOStat == ENABLE)
			GPTx->GTIOR |= GPT0_GTIOR00_OBE_Msk;		    
		else
			GPTx->GTIOR &= ~GPT0_GTIOR00_OBE_Msk;  
	}
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCA/GTIOCB IO status when terminal Prohibited Use
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Chann: 
  *            PWM_GTIOCA, PWM_GTIOCB
  * @param  IOStat:  
  *            PWM_GTIOCx_ProhibOut 
  *            PWM_GTIOCx_HiZ       
  *            PWM_GTIOCx_Low       
  *            PWM_GTIOCx_High      
  * @retval None
  */
void PWM_GTIOCxIO_ProhibUseStat(GPT_Type * GPTx,uint8_t Chann,GPTDis_FunState IOStat)
{
	
	PWM_UnLock(GPTx);								//дĴ

	if(Chann == PWM_GTIOCA)
	{
		GPTx->GTIOR &= ~GPT0_GTIOR00_OADF_Msk;   
		GPTx->GTIOR |= IOStat << GPT0_GTIOR00_OADF_Pos;  
	}
  else
	{
		GPTx->GTIOR &= ~GPT0_GTIOR00_OBDF_Msk;   
		GPTx->GTIOR |= IOStat << GPT0_GTIOR00_OBDF_Pos;		
	}
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCA/GTIOCB IO input filter control
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Chann: 
  *            PWM_GTIOCA, PWM_GTIOCB
  * @param  IOStat: 
  *            ENABLEʹGTIOCx IO˲
  *            DISABLEֹGTIOCx IO˲   
  * @retval None
  */
void PWM_GTIOCxIO_InFilter(GPT_Type * GPTx,uint8_t Chann,FunctionalState IOStat)
{
	
	PWM_UnLock(GPTx);								//дĴ

	if(Chann == PWM_GTIOCA)
	{
		if(IOStat == ENABLE)
			GPTx->GTIOR |= GPT0_GTIOR00_NFAEN_Msk;	     
		else
			GPTx->GTIOR &= ~GPT0_GTIOR00_NFAEN_Msk;   
	}
  else
	{
		if(IOStat == ENABLE)
			GPTx->GTIOR |= GPT0_GTIOR00_NFBEN_Msk;		    
		else
			GPTx->GTIOR &= ~GPT0_GTIOR00_NFBEN_Msk;  
	}
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTIOCA/GTIOCB IO input filter frequency select
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  Chann: 
  *            PWM_GTIOCA, PWM_GTIOCB
  * @param  IOStat:  
  *            Freq_Div1 1Ƶ
  *            Freq_Div4 4Ƶ
  *            Freq_Div16 Ƶ
  *            Freq_Div64 64Ƶ
  * @retval None
  */
void PWM_GTIOCxIOIn_FilterFreq(GPT_Type * GPTx,uint8_t Chann,GPTFiltDiv_FunState IOStat)
{
	
	PWM_UnLock(GPTx);								//дĴ

	if(Chann == PWM_GTIOCA)
	{
		GPTx->GTIOR &= ~GPT0_GTIOR00_NFCSA_Msk;  
		GPTx->GTIOR |= IOStat << GPT0_GTIOR00_NFCSA_Pos;	     
	}
  else
	{
		GPTx->GTIOR &= ~GPT0_GTIOR00_NFCSB_Msk;  
		GPTx->GTIOR |= IOStat << GPT0_GTIOR00_NFCSB_Pos;	
	}
	
	PWM_Lock(GPTx);								//ֹдĴ		
}


/**
  * @brief  PWM interrupt request deal
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  IOStat:  
  *            Int_GtcACmpCap 
  *             .........
  *            Int_GtcRSwtoothDouble 
  * @param  INTStat: 
  *            ENABLEʹж
  *            DISABLEֹж   
  * @retval None
  */
void PWM_Interrupt_Req(GPT_Type * GPTx,GPTINT_FunState INTType,FunctionalState INTStat)
{
	
	PWM_UnLock(GPTx);								//дĴ

	if(INTStat == ENABLE)
	{
		if(INTType == Int_GtcRSwtoothDouble)
			GPTx->GTINTAD |= 3<< Int_GtcRSwtoothOv;
		else
			GPTx->GTINTAD |= 1<< INTType;      
	}
  else
	{
		if(INTType == Int_GtcRSwtoothDouble)
			GPTx->GTINTAD &= ~(3<< Int_GtcRSwtoothOv);
		else
			GPTx->GTINTAD &= ~(1<< INTType);  
	}
	
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM Group A B C D disable select
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  IOStat:  
  *            Int_Dis_GroupA  //A
  *            Int_Dis_GroupB  //B
  *            Int_Dis_GroupC  //C
  *            Int_Dis_GroupD  //D
  * @retval None
  */
void PWM_DisRequest_GroupSelect(GPT_Type * GPTx,GPTINT_FunState INTType)
{
	uint32_t Temp;
	
	PWM_UnLock(GPTx);								//дĴ
   
	Temp = GPTx->GTINTAD;
	Temp &= ~GPT0_GTINTAD00_GRP_Msk;
	Temp |= (INTType - Int_Dis_GroupA) << GPT0_GTINTAD00_GRP_Pos; 
	GPTx->GTINTAD = Temp; 
	
	PWM_Lock(GPTx);								//ֹдĴ		
}


/**
  * @brief  PWM priority disable select
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  INTType:  
  *            Int_Dis_GRPABH  //
  *            Int_Dis_GRPABL  //
  * @retval None
  */
void PWM_DisPriority_Select(GPT_Type * GPTx,GPTINT_FunState INTType)
{
	
	PWM_UnLock(GPTx);								//дĴ
  if(INTType == Int_Dis_GRPABH)
	{
		GPTx->GTINTAD |= GPT0_GTINTAD00_GRPABH_Msk; 
		GPTx->GTINTAD &= ~GPT0_GTINTAD00_GRPABL_Msk;
	}
	else
	{
		GPTx->GTINTAD |= GPT0_GTINTAD00_GRPABL_Msk; 
		GPTx->GTINTAD &= ~GPT0_GTINTAD00_GRPABH_Msk;
	}
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GPTR buffer select
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  BufCtrl:  
  *            ENABLE
  *            DISABLEⲿ
  * @param  BufType:  
  *            GPT_BuffNone:BUFFER
  *            GPT_BuffSingle:BUFFER
  *            GPT_BuffDouble:˫BUFFER
  * @retval None
  */
void PWM_PRBuffer_Select(GPT_Type * GPTx,FunctionalState BufCtrl,GPTBuff_FunState BufType)
{
	
	PWM_UnLock(GPTx);								//дĴ
  if(BufCtrl == ENABLE)
	{
		GPTx->GTBER &= ~GPT0_GTBER00_BD1_Msk;
		GPTx->GTBER |= BufType<<GPT0_GTBER00_PR_Pos;
	}
	else
	{
		GPTx->GTBER |= GPT0_GTBER00_BD1_Msk;
	}
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  PWM GTCCR buffer select
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @param  BufCtrl:  
  *            ENABLE
  *            DISABLEⲿ
  * @param  BufType:  
  *            GPT_BuffNone:BUFFER
  *            GPT_BuffSingle:BUFFER
  *            GPT_BuffDouble:˫BUFFER
  * @retval None
  */
void PWM_GTCCRBuffer_Select(GPT_Type * GPTx,FunctionalState BufCtrl,GPTBuff_FunState BufAType,GPTBuff_FunState BufBType)
{
	
	PWM_UnLock(GPTx);								//дĴ
  if(BufCtrl == ENABLE)
	{
		GPTx->GTBER &= ~GPT0_GTBER00_BD0_Msk;
		GPTx->GTBER |= BufAType<<GPT0_GTBER00_CCRA_Pos;
		GPTx->GTBER |= BufBType<<GPT0_GTBER00_CCRB_Pos;
	}
	else
	{
		GPTx->GTBER |= GPT0_GTBER00_BD0_Msk;
	}
	PWM_Lock(GPTx);								//ֹдĴ		
}

/**
  * @brief  set PWM feedback condition
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17
  * @param  FbStat: 
  *            ENABLE
  *            DISABLEⲿ
  * @retval None
  */
void PWM_FB_Ctrl(GPT_Type * GPTx,FunctionalState FbStat)
{
	uint32_t Retstat = 0;
	PWM_UnLock(GPTx);								//дĴ
	
	if(FbStat == ENABLE)
		GPT10OPS->OPSCR |= GPT1_OPSCR_FB_Msk;
	else
		GPT10OPS->OPSCR &= ~GPT1_OPSCR_FB_Msk;

	PWM_Lock(GPTx);								//ֹдĴ	

}


/**
  * @brief  return PWM status
  * @param  GPTx: 
	*            GPT00,GPT01,GPT02,GPT03
	*            GPT10 ~ GPT17  
  * @retval PWM status
  */
uint32_t PWM_Status(GPT_Type * GPTx)
{
	uint32_t Retstat = 0;
	PWM_UnLock(GPTx);								//дĴ
	Retstat =	GPTx-> GTST;
	PWM_Lock(GPTx);								//ֹдĴ	

	return(Retstat);	
}



void GPT_Init(GPT_Type * GPTx,GPT_InitTypeDef *GPT_InitStruct)
{
	
	if((GPTx == GPT00)||(GPTx == GPT01)||(GPTx == GPT02)||(GPTx == GPT03))
		CGC_PERAPeriphClockCmd(CGC_PERAPeriph_GPT0,ENABLE);
	else
		CGC_PERAPeriphClockCmd(CGC_PERAPeriph_GPT1,ENABLE);
	
//	if(GPT_InitStruct->PwmMod == )
	PWM_WaveMode(GPTx,GPT_InitStruct->PwmMod); //set work mode
	PWM_TimerFrequency(GPTx,GPT_InitStruct->PwmFreqDiv); //set module frequency
	
	PWM_UnLock(GPTx);								//дĴ
	GPTx->GTCNT = 0;
	GPTx->GTPR = GPT_InitStruct->Period-1;   //
	GPTx->GTPBR = GPT_InitStruct->Period-1;
	
	GPTx->GTUDDTYC = GPT_InitStruct->PwmCountDir | (GPT_InitStruct-> PwmCountDirF <<GPT0_GTUDDTYC00_UDF_Pos)
	                 | (GPT_InitStruct-> GptAFDuty <<GPT0_GTUDDTYC00_OADTY_Pos)| (GPT_InitStruct-> GptAFDutyCtrl <<GPT0_GTUDDTYC00_OADTYF_Pos)
	                 | (GPT_InitStruct-> GptBFDuty <<GPT0_GTUDDTYC00_OBDTY_Pos)| (GPT_InitStruct-> GptBFDutyCtrl <<GPT0_GTUDDTYC00_OBDTYF_Pos) ;
	
  GPTx->GTIOR =  GPT_InitStruct->GptIOAFunc | (GPT_InitStruct-> GptIOBFunc <<GPT0_GTIOR00_GTIOB_Pos) //
	                 | (GPT_InitStruct-> GptAOutCtrl <<GPT0_GTIOR00_OAE_Pos)| (GPT_InitStruct-> GptBOutCtrl <<GPT0_GTIOR00_OBE_Pos)  // ʹ
									 | (GPT_InitStruct-> PwmAFilt <<GPT0_GTIOR00_NFAEN_Pos)	| (GPT_InitStruct-> PwmBFilt <<GPT0_GTIOR00_NFBEN_Pos)		//˲ʹ
									 | (GPT_InitStruct-> GptAFiltDiv <<GPT0_GTIOR00_NFCSA_Pos)	| (GPT_InitStruct-> GptBFiltDiv <<GPT0_GTIOR00_NFCSB_Pos)  //˲Ƶ
									 | (GPT_InitStruct-> GptIOADefaultS <<GPT0_GTIOR00_OAHLD_Pos)| (GPT_InitStruct-> GptIOBDefaultS <<GPT0_GTIOR00_OBHLD_Pos);		 //ֹͣ״̬	
	
	if(GPT_InitStruct->GptPrBuff == 0)
		GPTx->GTBER |= GPT0_GTBER00_BD1_Msk;
	else
	{
		GPTx->GTBER &= ~GPT0_GTBER00_BD1_Msk;
		GPTx->GTBER |= GPT_InitStruct->GptPrBuff;
	}
	if((GPT_InitStruct->GptABuff == 0)&&(GPT_InitStruct->GptBBuff == 0))
		GPTx->GTBER |= GPT0_GTBER00_BD0_Msk; 
	else
	{
	  GPTx->GTBER &= ~GPT0_GTBER00_BD0_Msk;
		GPTx->GTBER |= GPT_InitStruct->GptABuff | GPT_InitStruct->GptBBuff;		
	}
	
	GPTx->GTCCRA = GPT_InitStruct->DutyA-1;
	GPTx->GTCCRB = GPT_InitStruct->DutyB-1;
	GPTx->GTCCRC = GPT_InitStruct->DutyA-1;
	GPTx->GTCCRD = GPT_InitStruct->DutyA-1;
	GPTx->GTCCRE = GPT_InitStruct->DutyB-1;
	GPTx->GTCCRF = GPT_InitStruct->DutyB-1;

	GPTx->GTPSR |= GPT_InitStruct->PwmStopSrc;
	GPTx->GTCSR |= GPT_InitStruct->PwmClrSrc;	
	GPTx->GTSSR = 1<< GPT_InitStruct->PwmSrc; //Դѡ
	GPT00->GTCR |= GPT0_GTCR00_CST_Msk;   //ִм

	PWM_Lock(GPTx);								//ֹдĴ
	
	PWM_Start(GPT00);
}






