#include "trng.h"

/***********************************************************************************************************************
* Function Name: TRNG_Init
* @brief  Initializes the RNG module.
* @param  None
* @return None
***********************************************************************************************************************/
void TRNG_Init(trng_div_t div_cnt)
{
    /* Make sure TRNG Operation Clock under 4MHz! */
    MISC->TRNGCKS &= MISC_TRNGCKS_CKS_Msk;
    MISC->TRNGCKS = div_cnt;

//    CGC->PERA |= CGC_PERA_TRNGEN_Msk; //TRNG CLK EN
		CGC_PERAPeriphClockCmd(CGC_PERAPeriph_TRNG,ENABLE);
}
/***********************************************************************************************************************
* Function Name: TRNG_DeInit
* @brief  DeInitializes the RNG module.
* @param  None
* @return None
***********************************************************************************************************************/
void TRNG_DeInit(void)
{
    /* Disable the RNG generator */
    TRNG->SCR &= ~TRNG_SCR_TRNGEN_Msk;
    
    CGC->PERA &= ~CGC_PERA_TRNGEN_Msk; //TRNG CLK EN
}

/***********************************************************************************************************************
* Function Name: TRNG_GetRandomNumber
* @brief  Get RandomNumber.
* @param  None
* @return 32 bits RandomNumber
* @note   Generating a 32-bit random number seed takes approximately 112 s.
***********************************************************************************************************************/
uint32_t TRNG_GetRandomNumber(void)
{
    uint32_t i, val;

    /* Enable the RNG generator */
    TRNG->SCR |= TRNG_SCR_TRNGEN_Msk;

    TRNG->SCR |= TRNG_SCR_TRNGST_Msk; 
    while((TRNG->SCR & TRNG_SCR_TRNGRDY_Msk) == 0) {}

    val = 0x00000000U;
    for (i=0; i<4; i++)
    {
        val = val << 8 | TRNG->SDR;
    }
    /* Disable the RNG generator */
    TRNG->SCR &= ~(TRNG_SCR_TRNGEN_Msk);

    return val;
}






