#include "iica_demo.h"
volatile uint8_t   g_iica_tx_end;             /* iica0 send data end */
volatile uint8_t   g_iica_rx_end;             /* iica0 receive data end */
uint16_t  g_iica_rx_cnt;                     /* iica1 receive data count */

#define _80_IICA_ADDRESS_COMPLETE                     (0x80U)
#define _00_IICA_MASTER_FLAG_CLEAR                    (0x00U)

void Iica0_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStruct = {0};
    I2CA_InitTypeDef I2CA_InitStructure;

		GPIO_PinAFConfig(GPIO_PORTD, GPIO_Pin_0, GROUP_AF_SCLA0);
		GPIO_PinAFConfig(GPIO_PORTD, GPIO_Pin_1, GROUP_AF_SDAA0);
    /*SCLA0 GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_0 ;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;
    GPIO_InitStruct.GPIO_PuPd   = GPIO_PuPd_UP;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_OD;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORTD, &GPIO_InitStruct);

    /*SDAA0 GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_1;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;
    GPIO_InitStruct.GPIO_PuPd   = GPIO_PuPd_UP;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_OD;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORTD, &GPIO_InitStruct);

    I2CA_InitStructure.I2C_ClockSpeed = 100000;
    I2CA_InitStructure.I2C_Ack = I2CA_Ack_Enable;
    I2CA_InitStructure.I2C_DutyCycle = 50;
    I2CA_InitStructure.I2C_OwnAddress = 0xA0;
    I2CA_InitStructure.I2C_Mode = I2CA_Mode_SMBusMaster;
    I2CA_Init(I2CA0,&I2CA_InitStructure);
#ifdef I2CA_USING_INTERRUPT
    ISR_Register(IICA0_IRQn, iica0_interrupt);
#endif
}
#if defined(BAT32G1XX_80PIN) || defined(BAT32G1XX_100PIN)
void Iica1_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStruct = {0};
    I2CA_InitTypeDef I2CA_InitStructure;

		GPIO_PinAFConfig(GPIO_PORTB, GPIO_Pin_11, GROUP_AF_SCLA1);
		GPIO_PinAFConfig(GPIO_PORTB, GPIO_Pin_10, GROUP_AF_SDAA1);
    /*SCLA1 GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_11;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;
    GPIO_InitStruct.GPIO_PuPd   = GPIO_PuPd_UP;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_OD;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORTB, &GPIO_InitStruct);

    /*SDAA1 GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_10;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;
    GPIO_InitStruct.GPIO_PuPd   = GPIO_PuPd_UP;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_OD;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORTB, &GPIO_InitStruct);

    I2CA_InitStructure.I2C_ClockSpeed = 100000;
    I2CA_InitStructure.I2C_Ack = I2CA_Ack_Enable;
    I2CA_InitStructure.I2C_DutyCycle = 50;
    I2CA_InitStructure.I2C_OwnAddress = 0xA0;
    I2CA_InitStructure.I2C_Mode = I2CA_Mode_SMBusMaster;
    I2CA_Init(I2CA1,&I2CA_InitStructure);
#ifdef I2CA_USING_INTERRUPT
    ISR_Register(IICA1_IRQn, iica1_interrupt);
#endif
}
#endif
/***********************************************************************************************************************
* Function Name: IICA0_MasterWrite
* @brief  This function starts to write data by interrupt as master mode.
* @param  adr - send address
* @param  tx_buf - transfer buffer pointer
* @param  tx_num - buffer size
* @param  wait :wait for start condition
* @return status
*             - MD_OK or MD_ERROR1 or MD_ERROR2
***********************************************************************************************************************/
int IICA0_MasterWrite(uint8_t adr, uint8_t *tx_buf, uint16_t tx_num, uint8_t wait)
{
    I2CA_Status status = OK;

	
    if (I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_BUSBSY) != RESET)
    {
        status = BUSY;
    }
    else
    {
			I2CA_WriteByte(I2CA0,adr);// write address
        I2CA_GenerateSTART(I2CA0);

        /* Wait */
        while (wait--)
        {
            ;
        }

        if (RESET == I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_STD))   //if start condition does not occur
        {
            status = NSTART;
        }

        /* Set parameter */
        pData.data = tx_buf;
        pData.len = tx_num;
        pData.flag = INT_IDLE;
        g_iica_tx_end = 0;

        adr &= (uint8_t)0xFE;    // set write mode           
		I2CA_WriteByte(I2CA0,adr);// write address
    }

    return (status);
}
#if defined(BAT32G1XX_80PIN) || defined(BAT32G1XX_100PIN)	
/***********************************************************************************************************************
* Function Name: IICA1_MasterWrite
* @brief  This function starts to write data by interrupt as master mode.
* @param  adr - send address
* @param  tx_buf - transfer buffer pointer
* @param  tx_num - buffer size
* @param  wait :wait for start condition
* @return status
*             - MD_OK or MD_ERROR1 or MD_ERROR2
***********************************************************************************************************************/
int IICA1_MasterWrite(uint8_t adr, uint8_t *tx_buf, uint16_t tx_num, uint8_t wait)
{
    I2CA_Status status = OK;

	
    if (I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_BUSBSY) != RESET)
    {
        status = BUSY;
    }
    else
    {
        I2CA_GenerateSTART(I2CA1);

        /* Wait */
        while (wait--)
        {
            ;
        }

        if (RESET == I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_STD))   //if start condition does not occur
        {
            status = NSTART;
        }

        /* Set parameter */
        pData.data = tx_buf;
        pData.len = tx_num;
        pData.flag = INT_IDLE;
        g_iica_tx_end = 0;

        adr &= (uint8_t)0xFE;    // set write mode           
		I2CA_WriteByte(I2CA1,adr);// write address
    }

    return (status);
}
#endif
/***********************************************************************************************************************
* Function Name: IICA_MasterRead
* @brief  This function starts to read data as master mode.
* @param  adr - receive address
* @param  rx_buf - transfer buffer pointer
* @param  rx_num - buffer size
* @param  wait :wait for start condition
* @return status
*             - MD_OK or MD_ERROR1 or MD_ERROR2
***********************************************************************************************************************/
int IICA0_MasterReceive(uint8_t adr, uint8_t *const rx_buf, uint16_t rx_num, uint8_t wait)
{
    I2CA_Status status = OK;


    NVIC_DisableIRQ(IICA0_IRQn);   // disable INTIICA0 interrupt

    if (I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_BUSBSY) != RESET)
    {
        /* Check bus busy */
        NVIC_EnableIRQ(IICA0_IRQn);   // enable INTIICA0 interrupt
        status = BUSY;
    }
    else
    {
        I2CA_GenerateSTART(I2CA0);     //generate a start condition
        NVIC_EnableIRQ(IICA0_IRQn);   // enable INTIICA0 interrupt

        /* Wait */
        while (wait--)
        {
            ;
        }

        if (RESET == I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_STD))   //check no start condition occurs
        {
            status = NSTART;
        }

        /* Set parameter */
        pData.data = rx_buf;
        pData.len = rx_num;
        pData.flag = INT_IDLE;

        g_iica_rx_end = 0;
        g_iica_rx_cnt = 0;
        adr |= (uint8_t)0x01;       // set receive mode
		I2CA_WriteByte(I2CA0,adr);// write address
    }

    return (status);
}
#if defined(BAT32G1XX_80PIN) || defined(BAT32G1XX_100PIN)
/***********************************************************************************************************************
* Function Name: IICA1_MasterRead
* @brief  This function starts to read data as master mode.
* @param  adr - receive address
* @param  rx_buf - transfer buffer pointer
* @param  rx_num - buffer size
* @param  wait :wait for start condition
* @return status
*             - MD_OK or MD_ERROR1 or MD_ERROR2
***********************************************************************************************************************/
int IICA1_MasterReceive(uint8_t adr, uint8_t *const rx_buf, uint16_t rx_num, uint8_t wait)
{
    I2CA_Status status = OK;


    NVIC_DisableIRQ(IICA1_IRQn);   // disable INTIICA0 interrupt

    if (I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_BUSBSY) != RESET)
    {
        /* Check bus busy */
        NVIC_EnableIRQ(IICA1_IRQn);   // enable INTIICA0 interrupt
        status = BUSY;
    }
    else
    {
        I2CA_GenerateSTART(I2CA1);     //generate a start condition
        NVIC_EnableIRQ(IICA1_IRQn);   // enable INTIICA0 interrupt

        /* Wait */
        while (wait--)
        {
            ;
        }

        if (RESET == I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_STD))   //check no start condition occurs
        {
            status = NSTART;
        }

        /* Set parameter */
        pData.data = rx_buf;
        pData.len = rx_num;
        pData.flag = INT_IDLE;

        g_iica_rx_end = 0;
        g_iica_rx_cnt = 0;
        adr |= (uint8_t)0x01;       // set receive mode
		I2CA_WriteByte(I2CA1,adr);// write address
    }

    return (status);
}
#endif
/***********************************************************************************************************************
* Function Name: IICA_Clear_Wakeup
* @brief  This function clears the WUPn bit of IICCTL01 register.
* @param  None
* @return None
***********************************************************************************************************************/
void IICA_Clear_Wakeup(I2CASelect_TypeDef I2CAx)
{
	IICA_TypeDef *I2CA_Instance;
	if(I2CAx == I2CA0)	
	{
		I2CA_Instance = (IICA_TypeDef *)IICA0;
	}
#if defined(BAT32G1XX_80PIN) || defined(BAT32G1XX_100PIN)
	else if(I2CAx == I2CA1)
	{
		I2CA_Instance = (IICA_TypeDef *)IICA1;
	}
#endif
	I2CA_Instance->IICCTL1 &= ~0x80; // WUP = 0
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
}



/***********************************************************************************************************************
* Function Name: iica_master_send_end
* @brief  This function is a callback function when IICA0 finishes master transmission.
* @param  None
* @return None
***********************************************************************************************************************/
static void iica_master_send_end(I2CASelect_TypeDef I2CAx)
{
	if(I2CAx == I2CA0)	
	{
    IICA0->IICCTL0 |= IICA0_IICCTL00_SPT_Msk;             // stop condition is generated

    while (0U == I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_SPD));   // stop condition is detected
	}
	#if defined(BAT32G1XX_80PIN) || defined(BAT32G1XX_100PIN)
	else if(I2CAx == I2CA1)
	{
		IICA1->IICCTL0 |= IICA1_IICCTL10_SPT_Msk;             // stop condition is generated

    while (0U == I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_SPD));   // stop condition is detected
	}
	#endif
    g_iica_tx_end = 1;
}

/***********************************************************************************************************************
* Function Name: iica_master_recv_end
* @brief  This function is a callback function when IICA0 finishes master reception.
* @param  None
* @return None
***********************************************************************************************************************/
static void iica_master_recv_end(I2CASelect_TypeDef I2CAx)
{
	if(I2CAx == I2CA0)	
	{
    IICA0->IICCTL0 |= IICA0_IICCTL00_SPT_Msk;             /* stop condition is generated */

    while (0U == I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_SPD));   /* stop condition is detected */
	}
	#if defined(BAT32G1XX_80PIN) || defined(BAT32G1XX_100PIN)
	else if(I2CAx == I2CA1)
	{
		IICA1->IICCTL0 |= IICA1_IICCTL10_SPT_Msk;             /* stop condition is generated */

    while (0U == I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_SPD));   /* stop condition is detected */
	}
	#endif
    g_iica_rx_end = 1;
}


/***********************************************************************************************************************
* Function Name: iica0_masterhandler
* @brief   This function is IICA0 master handler.
* @param  None
* @return None
***********************************************************************************************************************/
static void iica_master_error(I2CA_Status flag)
{
    /* Start user code. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

static void iica0_masterhandler(void *msg)
{
    ATE_FRAME_t *pFrame = (ATE_FRAME_t *)msg;


    if ((I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_BUSBSY) == RESET) && (pFrame->len != 0U))// Control for communication
    {
        iica_master_error(NSTOP);//IICSTOP
    }
    else // Control for sended address
    {
        if ((pFrame->flag & _80_IICA_ADDRESS_COMPLETE) == 0U) //write address uncompleted
        {
            if (I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_ACK) != RESET)    //check iic received askd signal
            {
                pFrame->flag |= _80_IICA_ADDRESS_COMPLETE;  //write address completed

                if (I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_TRC))  //sendding status
                {  
					I2CA_SET_REG(I2CA0,IICCTL00,I2CA_WTIM_BIT);// interrupt request is generated at the ninth clock's falling edge
                    if (pFrame->len > 0)
                    {
						I2CA_WriteByte(I2CA0,*pFrame->data);
                        pFrame->data++;
                        pFrame->len--;
                    }
                    else
                    {
                        iica_master_send_end(I2CA0);//iic master send finished
                    }
                }
                else          //receiving status
                {
					I2CA_SET_REG(I2CA0,IICCTL00,I2CA_ACK_BIT); /* ACKE0 = 1U: enable acknowledgment */
					I2CA_CLEAR_REG(I2CA0,IICCTL00,I2CA_WTIM_BIT);// interrupt request is generated at the eighth clock's falling edge
					I2CA_SET_REG(I2CA0,IICCTL00,I2CA_WREL_BIT); // cancel wait 
				}
            }
            else
            {
                iica_master_error(NACK);//iic no ack
            }
        }
        else    //write address completed
        {
            if (I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_TRC))  // master in sending status
            {
                if ((0U == I2CA_GetFlagStaus(I2CA0,I2CA_STATUS_ACK)) && (pFrame->len != 0U))
                {
                    iica_master_error(NACK);//iic no ack
                }
                else
                {
                    if (pFrame->len > 0U)
                    {
						I2CA_WriteByte(I2CA0,*pFrame->data);
                        pFrame->data++;
                        pFrame->len--;
                    }
                    else
                    {
                        iica_master_send_end(I2CA0);//iic master send finished
                    }
                }
            }
            else     //master receive control
            {
                if (g_iica_rx_cnt < pFrame->len)
                {
                    *pFrame->data = IICA0->IICA;
                    pFrame->data++;
                    g_iica_rx_cnt++;

                    if (g_iica_rx_cnt == pFrame->len)
                    {
						I2CA_CLEAR_REG(I2CA0,IICCTL00,I2CA_ACK_BIT);// disable acknowledgment
						I2CA_SET_REG(I2CA0,IICCTL00,I2CA_WREL_BIT);// cancel wait
						I2CA_SET_REG(I2CA0,IICCTL00,I2CA_WTIM_BIT);//interrupt request is generated at the eighth clock's falling edge
                    }
                    else
                    {
						I2CA_SET_REG(I2CA0,IICCTL00,I2CA_WREL_BIT);// cancel wait						
                    }
                }
                else
                {
                    iica_master_recv_end(I2CA0);//iic received finished
                }
            }
        }
    }
}

#if defined(BAT32G1XX_80PIN) || defined(BAT32G1XX_100PIN)
static void iica1_masterhandler(void *msg)
{
    ATE_FRAME_t *pFrame = (ATE_FRAME_t *)msg;


    if ((I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_BUSBSY) == RESET) && (pFrame->len != 0U))// Control for communication
    {
        iica_master_error(NSTOP);//IICSTOP
    }
    else // Control for sended address
    {
        if ((pFrame->flag & _80_IICA_ADDRESS_COMPLETE) == 0U) //write address uncompleted
        {
            if (I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_ACK) != RESET)    //check iic received askd signal
            {
                pFrame->flag |= _80_IICA_ADDRESS_COMPLETE;  //write address completed

                if (I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_TRC))  //sendding status
                {  
					I2CA_SET_REG(I2CA1,IICCTL00,I2CA_WTIM_BIT);// interrupt request is generated at the ninth clock's falling edge
                    if (pFrame->len > 0)
                    {
						I2CA_WriteByte(I2CA1,*pFrame->data);
                        pFrame->data++;
                        pFrame->len--;
                    }
                    else
                    {
                        iica_master_send_end(I2CA1);//iic master send finished
                    }
                }
                else          //receiving status
                {
					I2CA_SET_REG(I2CA1,IICCTL00,I2CA_ACK_BIT); /* ACKE0 = 1U: enable acknowledgment */
					I2CA_CLEAR_REG(I2CA1,IICCTL00,I2CA_WTIM_BIT);// interrupt request is generated at the eighth clock's falling edge
					I2CA_SET_REG(I2CA1,IICCTL00,I2CA_WREL_BIT); // cancel wait 
				}
            }
            else
            {
                iica_master_error(NACK);//iic no ack
            }
        }
        else    //write address completed
        {
            if (I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_TRC))  // master in sending status
            {
                if ((0U == I2CA_GetFlagStaus(I2CA1,I2CA_STATUS_ACK)) && (pFrame->len != 0U))
                {
                    iica_master_error(NACK);//iic no ack
                }
                else
                {
                    if (pFrame->len > 0U)
                    {
						I2CA_WriteByte(I2CA1,*pFrame->data);
                        pFrame->data++;
                        pFrame->len--;
                    }
                    else
                    {
                        iica_master_send_end(I2CA1);//iic master send finished
                    }
                }
            }
            else     //master receive control
            {
                if (g_iica_rx_cnt < pFrame->len)
                {
                    *pFrame->data = IICA1->IICA;
                    pFrame->data++;
                    g_iica_rx_cnt++;

                    if (g_iica_rx_cnt == pFrame->len)
                    {
						I2CA_CLEAR_REG(I2CA1,IICCTL00,I2CA_ACK_BIT);// disable acknowledgment
						I2CA_SET_REG(I2CA1,IICCTL00,I2CA_WREL_BIT);// cancel wait
						I2CA_SET_REG(I2CA1,IICCTL00,I2CA_WTIM_BIT);//interrupt request is generated at the eighth clock's falling edge
                    }
                    else
                    {
						I2CA_SET_REG(I2CA1,IICCTL00,I2CA_WREL_BIT);// cancel wait						
                    }
                }
                else
                {
                    iica_master_recv_end(I2CA1);//iic received finished
                }
            }
        }
    }
}
#endif

void iica0_interrupt(void *msg)
{
    NVIC_ClearPendingIRQ(IICA0_IRQn);
    IICA_Clear_Wakeup(I2CA0);

    if ((IICA0->IICS0 & 0x80) == 0x80U)
    {
        iica0_masterhandler((void *)msg);
    }
    else
    {

    }
}
#if defined(BAT32G1XX_80PIN) || defined(BAT32G1XX_100PIN)
void iica1_interrupt(void *msg)
{
    NVIC_ClearPendingIRQ(IICA1_IRQn);
    IICA_Clear_Wakeup(I2CA1);

    if ((IICA1->IICS0 & 0x80) == 0x80U)
    {
//        iica1_masterhandler((void *)msg);
    }
    else
    {

    }
}
#endif
